@@ -17,7 +17,7 @@ use warnings;
 no warnings qw(recursion);
 
 use vars qw($VERSION $STRING_VERSION);
-$VERSION        = '4.001_047';
+$VERSION        = '4.001_049';
 $STRING_VERSION = $VERSION;
 ## no critic(BuiltinFunctions::ProhibitStringyEval)
 $VERSION = eval $VERSION;
@@ -233,9 +233,9 @@ sub nook_has_semantic_cause {
         ('@' . __FILE__ . ':' . __LINE__),
     <<'END_OF_LUA', 'i', $or_node);
     local recce, or_node = ...
-    local irl_id = recce.lmw_b:_or_node_irl(or_node)
+    local irl_id = recce.lmw_b:_or_node_nrl(or_node)
     local predot_position = recce.lmw_b:_or_node_position(or_node) - 1
-    local predot_isyid = recce.slg.g1:_irl_rhs(irl_id, predot_position)
+    local predot_isyid = recce.slg.g1:_nrl_rhs(irl_id, predot_position)
     return recce.slg.g1:_nsy_is_semantic(predot_isyid)
 END_OF_LUA
      return $result;
@@ -319,9 +319,9 @@ END_OF_LUA
     local recce, irlid = ...
     local slg = recce.slg
     local irl = slg.g1.irls[irlid]
-    local xbnf = irl.xbnf
-    if xbnf then
-        local blessing = xbnf.bless
+    local xpr = irl.xpr
+    if xpr then
+        local blessing = xpr.bless
         if blessing and string.sub(blessing, 1, 2) ~= '::' then
             return 'ok', blessing
         end
@@ -525,7 +525,7 @@ sub nid_sort_ix {
         ('@' . __FILE__ . ':' . __LINE__),
         <<'END_OF_LUA', 'i', $nid);
         local slr, nid = ...
-        local irl_id = slr.lmw_b:_or_node_irl(nid)
+        local irl_id = slr.lmw_b:_or_node_nrl(nid)
         return slr.slg.g1:_source_xrl(irl_id)
 END_OF_LUA
         return $result;
@@ -568,7 +568,7 @@ sub nid_rule_id {
         ('@' . __FILE__ . ':' . __LINE__),
     <<'END_OF_LUA', 'i', $nid);
     local recce, nid = ...
-    local irl_id = recce.lmw_b:_or_node_irl(nid)
+    local irl_id = recce.lmw_b:_or_node_nrl(nid)
     local xrl_id = recce.slg.g1:_source_xrl(irl_id)
     return xrl_id
 END_OF_LUA
@@ -686,7 +686,7 @@ sub nid_symbol_id {
         ('@' . __FILE__ . ':' . __LINE__),
     <<'END_OF_LUA',
         local slr, nid = ...
-        local irl_id = slr.lmw_b:_or_node_irl(nid)
+        local irl_id = slr.lmw_b:_or_node_nrl(nid)
         local g1g = slr.slg.g1
         local xrl_id = g1g:_source_xrl(irl_id)
         local lhs_id = g1g:rule_lhs(xrl_id)
@@ -1026,15 +1026,15 @@ sub glade_obtain {
             undef;
         my $symch_nidset = $choicepoint_powerset->nidset($asf, $symch_ix);
         my $choicepoint_nid = $symch_nidset->nid(0);
-        my $symch_rule_id = nid_rule_id($asf, $choicepoint_nid) // -1;
+        my $g1_symch_rule_id = nid_rule_id($asf, $choicepoint_nid) // -1;
 
         # Initial undef indicates no factorings omitted
-        my @factorings = ( $symch_rule_id, undef );
+        my @factorings = ( $g1_symch_rule_id, undef );
 
         # For a token
         # There will not be multiple factorings or nids,
         # it is assumed, for a token
-        if ( $symch_rule_id < 0 ) {
+        if ( $g1_symch_rule_id < 0 ) {
             my $base_nidset = Marpa::R3::Nidset->obtain( $asf, $choicepoint_nid );
             my $glade_id    = $base_nidset->id();
 
@@ -1043,7 +1043,7 @@ sub glade_obtain {
             push @factorings, [$glade_id];
             push @symches, \@factorings;
             next SYMCH;
-        } ## end if ( $symch_rule_id < 0 )
+        } ## end if ( $g1_symch_rule_id < 0 )
 
         my $symch = $choicepoint_powerset->nidset($asf, $symch_ix);
         my $nid_count = $symch->count();
@@ -1124,7 +1124,7 @@ END_OF_LUA
     return $l0_length;
 }
 
-sub Marpa::R3::ASF::glade_symbol_id {
+sub Marpa::R3::ASF::g1_glade_symbol_id {
     my ( $asf, $glade_id ) = @_;
     my $nidset_by_id = $asf->[Marpa::R3::Internal::ASF::NIDSET_BY_ID];
     my $nidset       = $nidset_by_id->[$glade_id];
@@ -1133,14 +1133,14 @@ sub Marpa::R3::ASF::glade_symbol_id {
     return nid_symbol_id($asf, $nid0);
 }
 
-sub Marpa::R3::ASF::symch_rule_id {
+sub Marpa::R3::ASF::g1_symch_rule_id {
     my ( $asf, $glade_id, $symch_ix ) = @_;
     my $glade = glade_obtain( $asf, $glade_id );
     my $symches = $glade->[Marpa::R3::Internal::Glade::SYMCHES];
     return if $symch_ix > $#{$symches};
     my ($rule_id) = @{ $symches->[$symch_ix] };
     return $rule_id;
-} ## end sub Marpa::R3::ASF::symch_rule_id
+}
 
 sub Marpa::R3::ASF::symch_factoring_count {
     my ( $asf, $glade_id, $symch_ix ) = @_;
@@ -1195,12 +1195,12 @@ sub Marpa::R3::Internal::ASF::glade_ambiguities {
     my $symch_count = $asf->glade_symch_count($glade);
     if ( $symch_count > 1 ) {
         my $literal      = $asf->glade_literal($glade);
-        my $symbol_id    = $asf->glade_symbol_id($glade);
-        my $display_form = $grammar->symbol_display_form($symbol_id);
+        my $symbol_id    = $asf->g1_glade_symbol_id($glade);
+        my $display_form = $grammar->g1_symbol_display_form($symbol_id);
         return [ [ 'symch', $glade, ] ];
     } ## end if ( $symch_count > 1 )
-    my $rule_id = $asf->symch_rule_id( $glade, 0 );
-    return [] if $rule_id < 0;       # no ambiguities if a token
+    my $g1_rule_id = $asf->g1_symch_rule_id( $glade, 0 );
+    return [] if $g1_rule_id < 0;       # no ambiguities if a token
 
     # ignore any truncation of the factorings
 
@@ -1345,8 +1345,8 @@ sub Marpa::R3::Internal::ASF::ambiguities_show {
 
             my ( undef, $glade ) = @{$ambiguity};
             my $symbol_display_form =
-                $grammar->symbol_display_form(
-                $asf->glade_symbol_id($glade) );
+                $grammar->g1_symbol_display_form(
+                $asf->g1_glade_symbol_id($glade) );
 
             my $l0_length = $asf->glade_L0_length($glade);
             my ( $g1_start, $g1_length )       = $asf->glade_g1_span($glade);
@@ -1381,13 +1381,13 @@ END_OF_LUA
                 ? "  There are $symch_count symches\n"
                 : "  There are $symch_count symches -- showing only the first $display_symch_count\n";
             SYMCH_IX: for my $symch_ix ( 0 .. $display_symch_count - 1 ) {
-                my $rule_id = $asf->symch_rule_id( $glade, $symch_ix );
-                if ( $rule_id < 0 ) {
+                my $g1_rule_id = $asf->g1_symch_rule_id( $glade, $symch_ix );
+                if ( $g1_rule_id < 0 ) {
                     $result .= "  Symch $symch_ix is a token\n";
                     next SYMCH_IX;
                 }
                 $result .= "  Symch $symch_ix is a rule: "
-                    . $grammar->rule_show($rule_id) . "\n";
+                    . $grammar->g1_rule_show($g1_rule_id) . "\n";
             } ## end SYMCH_IX: for my $symch_ix ( 0 .. $display_symch_count - 1 )
 
             next AMBIGUITY;
@@ -1406,8 +1406,8 @@ END_OF_LUA
                     $factoring_ix2 );
                 my $this_downglade = $these_downglades->[$factor_ix2];
                 my $symbol_display_form =
-                    $grammar->symbol_display_form(
-                    $asf->glade_symbol_id($first_downglade) );
+                    $grammar->g1_symbol_display_form(
+                    $asf->g1_glade_symbol_id($first_downglade) );
 
                 my ( $g1_start, $g1_length ) =
                     $asf->glade_g1_span($first_downglade);
@@ -1573,7 +1573,7 @@ sub Marpa::R3::Internal::ASF::Traverse::symbol_id {
     my $asf = $traverser->[Marpa::R3::Internal::ASF::Traverse::ASF];
     my $glade = $traverser->[Marpa::R3::Internal::ASF::Traverse::GLADE];
     my $glade_id = $glade->[Marpa::R3::Internal::Glade::ID];
-    return $asf->glade_symbol_id($glade_id);
+    return $asf->g1_glade_symbol_id($glade_id);
 }
 
 sub Marpa::R3::Internal::ASF::Traverse::rule_id {
@@ -1711,7 +1711,7 @@ sub Marpa::R3::ASF::dump_glade {
         $item_ix //= 0;
         push @lines,
               [ 0, undef, "Symbol #$item_ix "
-            . $grammar->symbol_display_form($asf->glade_symbol_id($glade_id))
+            . $grammar->g1_symbol_display_form($asf->g1_glade_symbol_id($glade_id))
             . " has $symch_count symches" ];
         $symch_indent += 2;
         $symch_choice = form_choice( $parent_choice, $item_ix );
@@ -1725,12 +1725,12 @@ sub Marpa::R3::ASF::dump_glade {
         if ( $symch_count > 1 ) {
             push @lines, [ $symch_indent , undef, "Symch #$current_choice" ];
         }
-        my $rule_id = $asf->symch_rule_id( $glade_id, $symch_ix );
+        my $rule_id = $asf->g1_symch_rule_id( $glade_id, $symch_ix );
         if ( $rule_id >= 0 ) {
             push @lines,
                 [
                 $symch_indent, $glade_id,
-                "Rule $rule_id: " . $grammar->rule_show($rule_id)
+                "Rule $rule_id: " . $grammar->g1_rule_show($rule_id)
                 ];
             for my $line (
                 @{ dump_factorings(
@@ -1798,9 +1798,9 @@ sub dump_terminal {
     # There can only be one symbol in a terminal and therefore only one factoring
     my $current_choice = $parent_choice;
     my $literal        = $asf->glade_literal($glade_id);
-    my $symbol_id    = $asf->glade_symbol_id($glade_id);
+    my $symbol_id    = $asf->g1_glade_symbol_id($glade_id);
     my $grammar = $asf->grammar();
-    my $display_form = $grammar->symbol_display_form($symbol_id);
+    my $display_form = $grammar->g1_symbol_display_form($symbol_id);
     return [0, $glade_id, qq{Symbol $display_form: "$literal"}];
 } ## end sub dump_terminal
 
