@@ -16,7 +16,7 @@ use strict;
 use warnings;
 
 use vars qw($VERSION $STRING_VERSION);
-$VERSION        = '4.001_047';
+$VERSION        = '4.001_049';
 $STRING_VERSION = $VERSION;
 ## no critic(BuiltinFunctions::ProhibitStringyEval)
 $VERSION = eval $VERSION;
@@ -56,19 +56,39 @@ sub Marpa::R3::Internal::MetaAST::Parse::line_column {
     return Marpa::R3::Internal::line_column($parse->{p_dsl}, $pos);
 }
 
+# Assign symbols, creating "ordinary" symbols if no symbol
+# already exists
+sub Marpa::R3::Internal::MetaAST::Parse::symbol_assign_ordinary {
+    my ( $parse, $symbol_name, $subg ) = @_;
+    my $wsym = $parse->{symbols}->{$subg}->{$symbol_name};
+    return $wsym if $wsym;
+    # say STDERR "symbol_assign_ordinary($symbol_name, $subg)";
+    my $symbol_data = {
+        dsl_form    => $symbol_name,
+        name_source => 'lexical'
+    };
+    $parse->xsy_assign( $symbol_name, $symbol_data );
+    $symbol_data = { xsy => $symbol_name };
+    $parse->symbol_names_set( $symbol_name, $subg,
+        $symbol_data );
+}
+
 sub ast_to_hash {
     my ($ast, $p_dsl) = @_;
+    my $xpr_ordinal = 0;
     my $hashed_ast = {};
 
     $hashed_ast->{meta_recce} = $ast->{meta_recce};
     bless $hashed_ast, 'Marpa::R3::Internal::MetaAST::Parse';
 
     $hashed_ast->{p_dsl} = $p_dsl;
-    $hashed_ast->{xbnf}->{L0} = {};
-    $hashed_ast->{xbnf}->{G1} = {};
-    $hashed_ast->{rules}->{L0} = [];
-    $hashed_ast->{rules}->{G1} = [];
-    my $g1_symbols = $hashed_ast->{symbols}->{G1} = {};
+    $hashed_ast->{xpr}->{l0} = {};
+    $hashed_ast->{xpr}->{g1} = {};
+    $hashed_ast->{rules}->{l0} = [];
+    $hashed_ast->{rules}->{g1} = [];
+    $hashed_ast->{lexeme_declarations} = {};
+    my $declarations = $hashed_ast->{lexeme_declarations};
+    my $g1_symbols = $hashed_ast->{symbols}->{g1} = {};
 
     my ( undef, undef, @statements ) = @{ $ast->{top_node} };
 
@@ -94,40 +114,141 @@ sub ast_to_hash {
           // $hashed_ast->{'first_lhs'};
         Marpa::R3::exception('No rules in SLIF grammar')
           if not defined $start_lhs;
-        my $augment_lhs = '[:start]';
+        my $augment_lhs = '[:start:]';
+        my $symbol_data = {
+            dsl_form    => $augment_lhs,
+            name_source => 'internal',
+        };
+        $hashed_ast->xsy_create( $augment_lhs, $symbol_data );
+        $hashed_ast->symbol_names_set( $augment_lhs, 'g1', { xsy => $augment_lhs } );
 
-        # description  => 'Internal G1 start symbol'
-        $hashed_ast->{'symbols'}->{'G1'}->{$augment_lhs} = {};
         my $rule_data = {
-            start => 0,
+            start  => 0,
             length => 0,
             lhs    => $augment_lhs,
             rhs    => [$start_lhs],
+            subkey => $xpr_ordinal++,
             action => '::first'
-          };
-        my $wrl= $hashed_ast->xbnf_create( $rule_data, 'G1' );
-        push @{ $hashed_ast->{rules}->{G1} }, $wrl;
-    } ## end sub Marpa::R3::Internal::MetaAST::start_rule_create
+        };
+        $hashed_ast->symbol_assign_ordinary($start_lhs, 'g1');
+        my $wrl = $hashed_ast->xpr_create( $rule_data, 'g1' );
+        push @{ $hashed_ast->{rules}->{g1} }, $wrl;
+    }
 
-    # add all the "ordinary" symbols, those with no
-    # special properties and whose name is their DSL
-    # form.
-    for my $grammar ( keys %{ $hashed_ast->{rules} } ) {
-        my $rules   = $hashed_ast->{rules}->{$grammar};
-        my $wsyms   = $hashed_ast->{symbols}->{$grammar};
-        my @symbols = map { $_->{lhs} } @{$rules};
-        push @symbols, map { @{ $_->{rhs} } } @{$rules};
-      SYM: for my $symbol_name (@symbols) {
-            next SYM if defined $wsyms->{$symbol_name};
+    # If the lexer is empty, create a fake one
+    # The fake lexer contains one rule.
+    # This rule discards everything it matches, but it
+    # never matches anything.
+    if ( not %{ $hashed_ast->{xpr}->{l0} } ) {
 
+        # the unicorn is a pattern which never matches
+        my $unicorn_class = '[[^\\d\\D]]';
+        my $unicorn;
+        {
+            local $Marpa::R3::Internal::SUBGRAMMAR = 'l0';
+            my $unicorn_symbol =
+              Marpa::R3::Internal::MetaAST::Symbol_List->char_class_to_symbol(
+                $hashed_ast, $unicorn_class );
+            $unicorn = $unicorn_symbol->name();
+        };
+        my $discard_lhs = '[:discard:]';
+        my $symbol_data = {
+            dsl_form    => $discard_lhs,
+            name_source => 'internal',
+        };
+        $hashed_ast->xsy_assign( $discard_lhs, $symbol_data );
+        $hashed_ast->symbol_names_set( $discard_lhs, 'l0',
+            { xsy => $discard_lhs } );
+        my $rule_data = {
+            start           => 0,
+            length          => 0,
+            lhs             => $discard_lhs,
+            rhs             => [$unicorn],
+            symbol_as_event => $unicorn,
+            subkey => $xpr_ordinal++,
+
+              # 'description' => 'Discard rule for <[[^\\d\\D]]>'
+        };
+        my $wrl = $hashed_ast->xpr_create( $rule_data, 'l0' );
+        push @{ $hashed_ast->{rules}->{l0} }, $wrl;
+    }
+
+    my %l0_lhs = ();
+    my %l0_rhs = ();
+    RULE: for my $rule (values %{$hashed_ast->{xpr}->{l0}}) {
+         my $lhs = $rule->{lhs};
+         $l0_lhs{$lhs} = 1;
+         $l0_rhs{$_} = 1 for @{$rule->{rhs}};
+         my $separator = $rule->{separator};
+         $l0_rhs{$separator} = 1 if $separator;
+    }
+    my %g1_lhs = ();
+    my %g1_rhs = ();
+    RULE: for my $rule (values %{$hashed_ast->{xpr}->{g1}}) {
+         my $lhs = $rule->{lhs};
+         $g1_lhs{$lhs} = 1;
+         $g1_rhs{$_} = 1 for @{$rule->{rhs}};
+         my $separator = $rule->{separator};
+         $g1_rhs{$separator} = 1 if $separator;
+    }
+
+
+    my %lexeme = ();
+    $lexeme{$_} = 'a lexeme in L0' for grep { not $l0_rhs{$_} } keys %l0_lhs;
+    $lexeme{$_} = 'a lexeme in G1' for grep { not $g1_lhs{$_} } keys %g1_rhs;
+    $lexeme{$_} = 'a declared lexeme' for keys %{$declarations};
+  LEXEME: for my $lexeme ( sort keys %lexeme ) {
+        $declarations->{$lexeme} //= {};
+        if ( $lexeme ne '[:discard:]' ) {
+            if ( not $l0_lhs{$lexeme} ) {
+                my $type = $lexeme{$lexeme};
+                Marpa::R3::exception(
+"<$lexeme> is $type, but is not on the LHS of any L0 rule\n",
+                    "    A lexeme must be the LHS of some L0 rule\n"
+                );
+            }
+            if ( $l0_rhs{$lexeme} ) {
+                my $type = $lexeme{$lexeme};
+                Marpa::R3::exception(
+                    "<$lexeme> is $type, but is on the RHS of an L0 rule\n",
+                    "    A lexeme must not be in the RHS of any L0 rule\n"
+                );
+            }
+            if ( $g1_lhs{$lexeme} ) {
+                my $type = $lexeme{$lexeme};
+                Marpa::R3::exception(
+                    "<$lexeme> is $type, but is on the LHS of a G1 rule\n",
+                    "    A lexeme cannot be the LHS of any G1 rule\n"
+                );
+            }
+            if ( not $g1_rhs{$lexeme} ) {
+                my $type = $lexeme{$lexeme};
+                Marpa::R3::exception(
+"<$lexeme> is $type, but is not on the RHS of any G1 rule\n",
+                    "    A lexeme must be in the RHS of at least one G1 rule\n"
+                );
+            }
+        }
+
+        {
+            my $lex_start_lhs = '[:lex_start:]';
             my $symbol_data = {
-                dsl_form    => $symbol_name,
-                name_source => 'lexical'
+                dsl_form    => $lex_start_lhs,
+                name_source => 'internal',
             };
-            $hashed_ast->xsy_assign( $symbol_name, $symbol_data );
-            $symbol_data = { xsy => $symbol_name };
-            $hashed_ast->symbol_names_set( $symbol_name, $grammar,
-                $symbol_data );
+            $hashed_ast->xsy_assign( $lex_start_lhs, $symbol_data );
+            $hashed_ast->symbol_names_set( $lex_start_lhs, 'l0',
+                { xsy => $lex_start_lhs } );
+
+            my $rule_data = {
+                start  => 0,
+                length => 0,
+                lhs    => $lex_start_lhs,
+                rhs    => [$lexeme],
+            };
+            $hashed_ast->symbol_assign_ordinary( $lex_start_lhs, 'l0' );
+            my $wrl = $hashed_ast->xpr_create( $rule_data, 'l0' );
+            push @{ $hashed_ast->{rules}->{l0} }, $wrl;
         }
     }
 
@@ -165,7 +286,7 @@ sub Marpa::R3::Internal::MetaAST::Proto_Alternative::combine {
 
 sub Marpa::R3::Internal::MetaAST::Parse::bless_hash_rule {
     my ( $parse, $hash_rule, $blessing, $naming, $original_lhs ) = @_;
-    return if (substr $Marpa::R3::Internal::SUBGRAMMAR, 0, 1) eq 'L0';
+    return if (substr $Marpa::R3::Internal::SUBGRAMMAR, 0, 1) eq 'l0';
 
     $naming //= $original_lhs;
     $hash_rule->{name} = $naming;
@@ -468,7 +589,7 @@ sub Marpa::R3::Internal::MetaAST_Nodes::adverb_item::evaluate {
 sub Marpa::R3::Internal::MetaAST_Nodes::default_rule::evaluate {
     my ( $values, $parse ) = @_;
     my ( $start, $length, undef, $op_declare, $raw_adverb_list ) = @{$values};
-    my $subgrammar = $op_declare->op() eq q{::=} ? 'G1' : 'L0';
+    my $subgrammar = $op_declare->op() eq q{::=} ? 'g1' : 'l0';
     my $adverb_list = $raw_adverb_list->evaluate($parse);
 
     # A default rule clears the previous default
@@ -477,11 +598,11 @@ sub Marpa::R3::Internal::MetaAST_Nodes::default_rule::evaluate {
 
     ADVERB: for my $key ( keys %{$adverb_list} ) {
         my $value = $adverb_list->{$key};
-        if ( $key eq 'action' and $subgrammar eq 'G1' ) {
+        if ( $key eq 'action' and $subgrammar eq 'g1' ) {
             $default_adverbs{$key} = $adverb_list->{$key};
             next ADVERB;
         }
-        if ( $key eq 'bless' and $subgrammar eq 'G1' ) {
+        if ( $key eq 'bless' and $subgrammar eq 'g1' ) {
             $default_adverbs{$key} = $adverb_list->{$key};
             next ADVERB;
         }
@@ -495,7 +616,7 @@ sub Marpa::R3::Internal::MetaAST_Nodes::default_rule::evaluate {
 sub Marpa::R3::Internal::MetaAST_Nodes::discard_default_statement::evaluate {
     my ( $data, $parse ) = @_;
     my ( $start, $length, $raw_adverb_list ) = @{$data};
-    local $Marpa::R3::Internal::SUBGRAMMAR = 'G1';
+    local $Marpa::R3::Internal::SUBGRAMMAR = 'g1';
 
     my $adverb_list = $raw_adverb_list->evaluate($parse);
     if ( exists $parse->{discard_default_adverbs} ) {
@@ -523,7 +644,7 @@ sub Marpa::R3::Internal::MetaAST_Nodes::discard_default_statement::evaluate {
 sub Marpa::R3::Internal::MetaAST_Nodes::lexeme_default_statement::evaluate {
     my ( $data, $parse ) = @_;
     my ( $start, $length, $raw_adverb_list ) = @{$data};
-    local $Marpa::R3::Internal::SUBGRAMMAR = 'G1';
+    local $Marpa::R3::Internal::SUBGRAMMAR = 'g1';
 
     my $adverb_list = $raw_adverb_list->evaluate($parse);
     if ( exists $parse->{lexeme_default_adverbs} ) {
@@ -555,7 +676,7 @@ sub Marpa::R3::Internal::MetaAST_Nodes::lexeme_default_statement::evaluate {
 sub Marpa::R3::Internal::MetaAST_Nodes::inaccessible_statement::evaluate {
     my ( $data, $parse ) = @_;
     my ( $start, $length, $inaccessible_treatment ) = @{$data};
-    local $Marpa::R3::Internal::SUBGRAMMAR = 'G1';
+    local $Marpa::R3::Internal::SUBGRAMMAR = 'g1';
 
     if ( exists $parse->{defaults}->{if_inaccessible} ) {
         my $problem_rule = $parse->substring( $start, $length );
@@ -578,11 +699,11 @@ sub Marpa::R3::Internal::MetaAST_Nodes::priority_rule::evaluate {
     my ( $start, $length, $raw_lhs, $op_declare, $raw_priorities ) =
         @{$values};
 
-    my $subgrammar = $op_declare->op() eq q{::=} ? 'G1' : 'L0';
-    my $xbnf_ordinal = 0;
+    my $subgrammar = $op_declare->op() eq q{::=} ? 'g1' : 'l0';
+    my $xpr_ordinal = 0;
 
     my $lhs = $raw_lhs->name($parse);
-    $parse->{'first_lhs'} //= $lhs if $subgrammar eq 'G1';
+    $parse->{'first_lhs'} //= $lhs if $subgrammar eq 'g1';
     local $Marpa::R3::Internal::SUBGRAMMAR = $subgrammar;
 
     my ( undef, undef, @priorities ) = @{$raw_priorities};
@@ -628,17 +749,18 @@ sub Marpa::R3::Internal::MetaAST_Nodes::priority_rule::evaluate {
             } ## end if ( not $eval_ok )
             my @rhs_names = @{ $proto_rule->{rhs} };
             my @mask      = @{ $proto_rule->{mask} };
-            if ( ( substr $subgrammar, 0, 1 ) eq 'L'
+            if ( ( substr $subgrammar, 0, 1 ) eq 'l'
                 and grep { !$_ } @mask )
             {
                 Marpa::R3::exception(
                     qq{hidden symbols are not allowed in lexical rules (rule's LHS was "$lhs")}
                 );
             }
+            $parse->symbol_assign_ordinary($_, $subgrammar) for $lhs, @rhs_names;
             my %hash_rule = (
                 start  => ( $alternative_ix ? $alternative_start  : $start ),
                 length => ( $alternative_ix ? $alternative_length : $length ),
-                subkey => ++$xbnf_ordinal,
+                subkey => ++$xpr_ordinal,
                 lhs    => $lhs,
                 rhs    => \@rhs_names,
                 mask   => \@mask,
@@ -687,7 +809,7 @@ sub Marpa::R3::Internal::MetaAST_Nodes::priority_rule::evaluate {
             if ( defined $action ) {
                 Marpa::R3::exception(
                     qq{actions not allowed in lexical rules (rule's LHS was "$lhs")}
-                ) if  ( substr $subgrammar, 0, 1 ) eq 'L';
+                ) if  ( substr $subgrammar, 0, 1 ) eq 'l';
                 $hash_rule{action} = $action;
             } ## end if ( defined $action )
 
@@ -695,7 +817,7 @@ sub Marpa::R3::Internal::MetaAST_Nodes::priority_rule::evaluate {
             if ( defined $rank ) {
                 Marpa::R3::exception(
                     qq{ranks not allowed in lexical rules (rule's LHS was "$lhs")}
-                ) if  ( substr $subgrammar, 0, 1 ) eq 'L';
+                ) if  ( substr $subgrammar, 0, 1 ) eq 'l';
                 $hash_rule{rank} = $rank;
             } ## end if ( defined $rank )
 
@@ -703,14 +825,14 @@ sub Marpa::R3::Internal::MetaAST_Nodes::priority_rule::evaluate {
             if ( defined $null_ranking ) {
                 Marpa::R3::exception(
                     qq{null-ranking allowed in lexical rules (rule's LHS was "$lhs")}
-                ) if  ( substr $subgrammar, 0, 1 ) eq 'L';
+                ) if  ( substr $subgrammar, 0, 1 ) eq 'l';
                 $hash_rule{null_ranking} = $null_ranking;
             } ## end if ( defined $rank )
 
             $blessing //= $default_adverbs->{bless};
             if (defined $blessing
                 and
-                 ( substr $subgrammar, 0, 1 ) eq 'L' 
+                 ( substr $subgrammar, 0, 1 ) eq 'l' 
                 )
             {
                 Marpa::R3::exception(
@@ -721,7 +843,7 @@ sub Marpa::R3::Internal::MetaAST_Nodes::priority_rule::evaluate {
 
             $parse->bless_hash_rule( \%hash_rule, $blessing, $naming, $lhs );
 
-            my $wrl = $parse->xbnf_create( \%hash_rule, $subgrammar );
+            my $wrl = $parse->xpr_create( \%hash_rule, $subgrammar );
             push @{$rules}, $wrl;
         } ## end for my $alternative (@alternatives)
         ## no critic(Subroutines::ProhibitExplicitReturnUndef)
@@ -758,16 +880,18 @@ sub Marpa::R3::Internal::MetaAST_Nodes::priority_rule::evaluate {
 
     # Default mask (all ones) is OK for this rule
     my @arg0_action = ();
-    @arg0_action = ( action => '::first' ) if $subgrammar eq 'G1';
+    @arg0_action = ( action => '::first' ) if $subgrammar eq 'g1';
 
     # Internal rule top priority rule for <$lhs>
+    $parse->symbol_assign_ordinary($lhs, $subgrammar);
     my @priority_rules = (
         {
             start => $start,
             length => $length,
             lhs   => $lhs,
             rhs   => [ $parse->prioritized_symbol( $lhs, 0 ) ],
-            subkey => ++$xbnf_ordinal,
+            precedence => -1,
+            subkey => ++$xpr_ordinal,
             @arg0_action,
         }
     );
@@ -779,12 +903,13 @@ sub Marpa::R3::Internal::MetaAST_Nodes::priority_rule::evaluate {
         length => $length,
         lhs   => $parse->prioritized_symbol( $lhs, $_ - 1 ),
         rhs   => [ $parse->prioritized_symbol( $lhs, $_ ) ],
-        subkey => ++$xbnf_ordinal,
+        precedence => ($_ - 1),
+        subkey => ++$xpr_ordinal,
         @arg0_action
       }
       for 1 .. $priority_count - 1;
   RULE: for my $priority_rule (@priority_rules) {
-        my $wrl = $parse->xbnf_create( $priority_rule, $subgrammar );
+        my $wrl = $parse->xpr_create( $priority_rule, $subgrammar );
         push @{$rules}, $wrl;
     }
 
@@ -796,7 +921,7 @@ sub Marpa::R3::Internal::MetaAST_Nodes::priority_rule::evaluate {
 
         my $current_exp = $parse->prioritized_symbol( $lhs, $priority );
         my @mask = @{ $rhs->{mask} };
-        if ( (  substr $subgrammar, 0, 1 ) eq 'L' and grep { !$_ } @mask )
+        if ( (  substr $subgrammar, 0, 1 ) eq 'l' and grep { !$_ } @mask )
         {
             Marpa::R3::exception(
                 'hidden symbols are not allowed in lexical rules (rules LHS was "',
@@ -807,7 +932,7 @@ sub Marpa::R3::Internal::MetaAST_Nodes::priority_rule::evaluate {
             lhs    => $current_exp,
             start  => $alternative_start,
             length => $alternative_length,
-            subkey => ++$xbnf_ordinal,
+            subkey => ++$xpr_ordinal,
             xrlid => $xrlid,
         );
         $new_xs_rule{mask} = \@mask;
@@ -855,7 +980,7 @@ sub Marpa::R3::Internal::MetaAST_Nodes::priority_rule::evaluate {
         if ( defined $action ) {
             Marpa::R3::exception(
                 qq{actions not allowed in lexical rules (rule's LHS was "$lhs")}
-            ) if  ( substr $subgrammar, 0, 1 ) eq 'L';
+            ) if  ( substr $subgrammar, 0, 1 ) eq 'l';
             $new_xs_rule{action} = $action;
         } ## end if ( defined $action )
 
@@ -863,7 +988,7 @@ sub Marpa::R3::Internal::MetaAST_Nodes::priority_rule::evaluate {
         if ( defined $null_ranking ) {
             Marpa::R3::exception(
                 qq{null-ranking not allowed in lexical rules (rule's LHS was "$lhs")}
-            ) if  ( substr $subgrammar, 0, 1 ) eq 'L';
+            ) if  ( substr $subgrammar, 0, 1 ) eq 'l';
             $new_xs_rule{null_ranking} = $null_ranking;
         } ## end if ( defined $rank )
 
@@ -871,13 +996,13 @@ sub Marpa::R3::Internal::MetaAST_Nodes::priority_rule::evaluate {
         if ( defined $rank ) {
             Marpa::R3::exception(
                 qq{ranks not allowed in lexical rules (rule's LHS was "$lhs")}
-            ) if  ( substr $subgrammar, 0, 1 ) eq 'L';
+            ) if  ( substr $subgrammar, 0, 1 ) eq 'l';
             $new_xs_rule{rank} = $rank;
         } ## end if ( defined $rank )
 
         $blessing //= $default_adverbs->{bless};
         if ( defined $blessing
-            and ( substr $subgrammar, 0, 1 ) eq 'L' )
+            and ( substr $subgrammar, 0, 1 ) eq 'l' )
         {
             Marpa::R3::exception(
                 'bless option not allowed in lexical rules (rules LHS was "',
@@ -896,10 +1021,12 @@ sub Marpa::R3::Internal::MetaAST_Nodes::priority_rule::evaluate {
         $next_priority = 0 if $next_priority >= $priority_count;
 
         my $next_exp = $parse->prioritized_symbol( $lhs, $next_priority);
+        $new_xs_rule{precedence} = $priority;
 
         if ( not scalar @arity ) {
             $new_xs_rule{rhs} = \@new_rhs;
-            my $wrl = $parse->xbnf_create( \%new_xs_rule, $subgrammar );
+            $parse->symbol_assign_ordinary($_, $subgrammar) for @new_rhs;
+            my $wrl = $parse->xpr_create( \%new_xs_rule, $subgrammar );
             push @{$rules}, $wrl;
             next RULE;
         }
@@ -932,8 +1059,9 @@ sub Marpa::R3::Internal::MetaAST_Nodes::priority_rule::evaluate {
             die qq{Unknown association type: "$assoc"};
         } ## end DO_ASSOCIATION:
 
+        $parse->symbol_assign_ordinary($_, $subgrammar) for @new_rhs;
         $new_xs_rule{rhs} = \@new_rhs;
-        my $wrl = $parse->xbnf_create( \%new_xs_rule, $subgrammar );
+        my $wrl = $parse->xpr_create( \%new_xs_rule, $subgrammar );
         push @{$rules}, $wrl;
     } ## end RULE: for my $working_rule (@working_rules)
     ## no critic(Subroutines::ProhibitExplicitReturnUndef)
@@ -945,10 +1073,10 @@ sub Marpa::R3::Internal::MetaAST_Nodes::empty_rule::evaluate {
     my ( $start, $length, $raw_lhs, $op_declare, $raw_adverb_list ) =
         @{$values};
 
-    my $subgrammar = $op_declare->op() eq q{::=} ? 'G1' : 'L0';
+    my $subgrammar = $op_declare->op() eq q{::=} ? 'g1' : 'l0';
 
     my $lhs = $raw_lhs->name($parse);
-    $parse->{'first_lhs'} //= $lhs if $subgrammar eq 'G1';
+    $parse->{'first_lhs'} //= $lhs if $subgrammar eq 'g1';
     local $Marpa::R3::Internal::SUBGRAMMAR = $subgrammar;
 
     my $xrlid = xrl_create($parse, {
@@ -1005,7 +1133,7 @@ sub Marpa::R3::Internal::MetaAST_Nodes::empty_rule::evaluate {
     if ( defined $action ) {
         Marpa::R3::exception(
             qq{actions not allowed in lexical rules (rule's LHS was "$lhs")}
-        ) if  ( substr $subgrammar, 0, 1 ) eq 'L';
+        ) if  ( substr $subgrammar, 0, 1 ) eq 'l';
         $rule{action} = $action;
     } ## end if ( defined $action )
 
@@ -1013,7 +1141,7 @@ sub Marpa::R3::Internal::MetaAST_Nodes::empty_rule::evaluate {
     if ( defined $null_ranking ) {
         Marpa::R3::exception(
             qq{null-ranking not allowed in lexical rules (rule's LHS was "$lhs")}
-        ) if  ( substr $subgrammar, 0, 1 ) eq 'L';
+        ) if  ( substr $subgrammar, 0, 1 ) eq 'l';
         $rule{null_ranking} = $null_ranking;
     } ## end if ( defined $null_ranking )
 
@@ -1021,13 +1149,13 @@ sub Marpa::R3::Internal::MetaAST_Nodes::empty_rule::evaluate {
     if ( defined $rank ) {
         Marpa::R3::exception(
             qq{ranks not allowed in lexical rules (rule's LHS was "$lhs")}
-        ) if  ( substr $subgrammar, 0, 1 ) eq 'L';
+        ) if  ( substr $subgrammar, 0, 1 ) eq 'l';
         $rule{rank} = $rank;
     } ## end if ( defined $rank )
 
     $blessing //= $default_adverbs->{bless};
     if ( defined $blessing
-        and ( substr $subgrammar, 0, 1 ) eq 'L' )
+        and ( substr $subgrammar, 0, 1 ) eq 'l' )
     {
         Marpa::R3::exception(
             qq{bless option not allowed in lexical rules (rule's LHS was "$lhs")}
@@ -1035,7 +1163,8 @@ sub Marpa::R3::Internal::MetaAST_Nodes::empty_rule::evaluate {
     }
     $parse->bless_hash_rule( \%rule, $blessing, $naming, $lhs );
 
-    my $wrl = $parse->xbnf_create( \%rule, $subgrammar );
+    $parse->symbol_assign_ordinary($lhs, $subgrammar);
+    my $wrl = $parse->xpr_create( \%rule, $subgrammar );
     # mask not needed
     push @{ $parse->{rules}->{$subgrammar} }, $wrl;
 
@@ -1170,14 +1299,15 @@ sub Marpa::R3::Internal::MetaAST_Nodes::discard_rule::evaluate {
     my ( $values, $parse ) = @_;
     my ( $start, $length, $symbol, $raw_adverb_list ) = @{$values};
 
-    local $Marpa::R3::Internal::SUBGRAMMAR = 'L0';
-    my $pseudo_lhs = '[:discard]';
-    $parse->symbol_names_set(
-        $pseudo_lhs,
-        'L0',
-        {   # description  => qq{Internal LHS for lexer discard}
-        }
-    );
+    local $Marpa::R3::Internal::SUBGRAMMAR = 'l0';
+    my $discard_lhs = '[:discard:]';
+    my $symbol_data = {
+        dsl_form    => $discard_lhs,
+        name_source => 'internal',
+    };
+    $parse->xsy_assign( $discard_lhs, $symbol_data );
+    $parse->symbol_names_set( $discard_lhs, 'l0', { xsy => $discard_lhs } );
+
     my $rhs         = $symbol->names($parse);
     my $discard_symbol = $rhs->[0];
     my $rhs_as_event         = $symbol->event_name($parse);
@@ -1205,15 +1335,15 @@ sub Marpa::R3::Internal::MetaAST_Nodes::discard_rule::evaluate {
 
     # Discard rule
     my %rule_hash = (
-        lhs => $pseudo_lhs,
+        lhs => $discard_lhs,
         rhs => [$discard_symbol],
         start => $start,
         length => $length,
         symbol_as_event => $rhs_as_event
     );
     $rule_hash{event} = $event if defined $event;
-    my $wrl = $parse->xbnf_create( \%rule_hash, 'L0' );
-    push @{ $parse->{rules}->{L0} }, $wrl;
+    my $wrl = $parse->xpr_create( \%rule_hash, 'l0' );
+    push @{ $parse->{rules}->{l0} }, $wrl;
     ## no critic(Subroutines::ProhibitExplicitReturnUndef)
     return undef;
 } ## end sub Marpa::R3::Internal::MetaAST_Nodes::discard_rule::evaluate
@@ -1224,10 +1354,10 @@ sub Marpa::R3::Internal::MetaAST_Nodes::quantified_rule::evaluate {
         $proto_adverb_list )
         = @{$values};
 
-    my $subgrammar = $op_declare->op() eq q{::=} ? 'G1' : 'L0';
+    my $subgrammar = $op_declare->op() eq q{::=} ? 'g1' : 'l0';
 
     my $lhs_name = $lhs->name($parse);
-    $parse->{'first_lhs'} //= $lhs_name if $subgrammar eq 'G1';
+    $parse->{'first_lhs'} //= $lhs_name if $subgrammar eq 'g1';
     local $Marpa::R3::Internal::SUBGRAMMAR = $subgrammar;
 
     my $quantifier_string = $quantifier->evaluate($parse);
@@ -1304,7 +1434,7 @@ sub Marpa::R3::Internal::MetaAST_Nodes::quantified_rule::evaluate {
     if ( defined $action ) {
         Marpa::R3::exception(
             qq{actions not allowed in lexical rules (rule's LHS was "$lhs")}
-        ) if  ( substr $subgrammar, 0, 1 ) eq 'L';
+        ) if  ( substr $subgrammar, 0, 1 ) eq 'l';
         $sequence_rule{action} = $action;
     } ## end if ( defined $action )
 
@@ -1312,7 +1442,7 @@ sub Marpa::R3::Internal::MetaAST_Nodes::quantified_rule::evaluate {
     if ( defined $null_ranking ) {
         Marpa::R3::exception(
             qq{null-ranking not allowed in lexical rules (rule's LHS was "$lhs")}
-        ) if  ( substr $subgrammar, 0, 1 ) eq 'L';
+        ) if  ( substr $subgrammar, 0, 1 ) eq 'l';
         $sequence_rule{null_ranking} = $null_ranking;
     } ## end if ( defined $null_ranking )
 
@@ -1320,20 +1450,22 @@ sub Marpa::R3::Internal::MetaAST_Nodes::quantified_rule::evaluate {
     if ( defined $rank ) {
         Marpa::R3::exception(
             qq{ranks not allowed in lexical rules (rule's LHS was "$lhs")}
-        ) if  ( substr $subgrammar, 0, 1 ) eq 'L';
+        ) if  ( substr $subgrammar, 0, 1 ) eq 'l';
         $sequence_rule{rank} = $rank;
     } ## end if ( defined $rank )
 
     $blessing //= $default_adverbs->{bless};
-    if ( defined $blessing and ( substr $subgrammar, 0, 1 ) eq 'L' )
+    if ( defined $blessing and ( substr $subgrammar, 0, 1 ) eq 'l' )
     {
         Marpa::R3::exception(
             qq{bless option not allowed in lexical rules (rule's LHS was "$lhs")}
         );
     }
+    $parse->symbol_assign_ordinary($_, $subgrammar) for $lhs_name, @{$sequence_rule{rhs}};
+    $parse->symbol_assign_ordinary($separator, $subgrammar) if defined $separator;
     $parse->bless_hash_rule( \%sequence_rule, $blessing, $naming, $lhs_name );
 
-    my $wrl = $parse->xbnf_create( \%sequence_rule, $subgrammar );
+    my $wrl = $parse->xpr_create( \%sequence_rule, $subgrammar );
     push @{ $parse->{rules}->{$subgrammar} }, $wrl;
     ## no critic(Subroutines::ProhibitExplicitReturnUndef)
     return undef;
@@ -1525,7 +1657,7 @@ sub Marpa::R3::Internal::MetaAST_Nodes::character_class::evaluate {
     my ( $values, $parse ) = @_;
     my ( $start, $length, $character_class ) = @{$values};
     my $subgrammar = $Marpa::R3::Internal::SUBGRAMMAR;
-    if ( ( substr $subgrammar, 0, 1 ) eq 'L' ) {
+    if ( ( substr $subgrammar, 0, 1 ) eq 'l' ) {
         return Marpa::R3::Internal::MetaAST::Symbol_List->char_class_to_symbol(
             $parse, $character_class );
     }
@@ -1533,7 +1665,7 @@ sub Marpa::R3::Internal::MetaAST_Nodes::character_class::evaluate {
     # If here, in G1
     # Character classes and strings always go into L0, for now
     my $lexer_symbol = do {
-        local $Marpa::R3::Internal::SUBGRAMMAR = 'L0';
+        local $Marpa::R3::Internal::SUBGRAMMAR = 'l0';
         Marpa::R3::Internal::MetaAST::Symbol_List->char_class_to_symbol( $parse,
             $character_class );
     };
@@ -1546,8 +1678,8 @@ sub Marpa::R3::Internal::MetaAST_Nodes::character_class::evaluate {
         rhs    => $lexical_rhs,
         mask   => [1],
     );
-    my $wrl = $parse->xbnf_create( \%lexical_rule, 'L0' );
-    push @{ $parse->{rules}->{L0} }, $wrl;
+    my $wrl = $parse->xpr_create( \%lexical_rule, 'l0' );
+    push @{ $parse->{rules}->{l0} }, $wrl;
     my $g1_symbol =
       Marpa::R3::Internal::MetaAST::Symbol_List->new($lexical_lhs);
     return $g1_symbol;
@@ -1566,7 +1698,7 @@ sub Marpa::R3::Internal::MetaAST_Nodes::single_quoted_string::evaluate {
 
     # If we are currently in a lexical grammar, the strings go there
     # If we are currently in G1, the strings always go into L0
-    my $lexical_grammar = $subgrammar eq 'G1' ? 'L0' : $subgrammar;
+    my $lexical_grammar = $subgrammar eq 'g1' ? 'l0' : $subgrammar;
 
     for my $char_class (
         map { '[' . ( quotemeta $_ ) . ']' . $flags } split //xms,
@@ -1581,19 +1713,23 @@ sub Marpa::R3::Internal::MetaAST_Nodes::single_quoted_string::evaluate {
         push @symbols, $symbol;
     } ## end for my $char_class ( map { '[' . ( quotemeta $_ ) . ']'...})
     my $list = Marpa::R3::Internal::MetaAST::Symbol_List->combine(@symbols);
-    return $list if $Marpa::R3::Internal::SUBGRAMMAR ne 'G1';
-    my $lexical_lhs       = $parse->internal_lexeme($string);
-    my $lexical_rhs       = $list->names($parse);
-    my %lexical_rule      = (
-        start => $start,
-        length => $length,
-        lhs  => $lexical_lhs,
-        rhs  => $lexical_rhs,
-        # description => "Internal rule for single-quoted string $string",
-        mask => [ map { ; 1 } @{$lexical_rhs} ],
-    );
-    my $wrl = $parse->xbnf_create( \%lexical_rule, 'L0' );
-    push @{ $parse->{rules}->{$lexical_grammar} }, $wrl;
+    return $list if $Marpa::R3::Internal::SUBGRAMMAR ne 'g1';
+    my $lexical_lhs = $parse->{lexeme_for_string}->{$string};
+    if (not defined $lexical_lhs) {
+        $lexical_lhs       = $parse->internal_lexeme($string);
+        $parse->{lexeme_for_string}->{$string}= $lexical_lhs;
+        my $lexical_rhs       = $list->names($parse);
+        my %lexical_rule      = (
+            start => $start,
+            length => $length,
+            lhs  => $lexical_lhs,
+            rhs  => $lexical_rhs,
+            # description => "Internal rule for single-quoted string $string",
+            mask => [ map { ; 1 } @{$lexical_rhs} ],
+        );
+        my $wrl = $parse->xpr_create( \%lexical_rule, 'l0' );
+        push @{ $parse->{rules}->{$lexical_grammar} }, $wrl;
+    }
     my $g1_symbol =
         Marpa::R3::Internal::MetaAST::Symbol_List->new($lexical_lhs);
     return $g1_symbol;
@@ -1703,7 +1839,7 @@ sub char_class_to_symbol {
 
 sub Marpa::R3::Internal::MetaAST::Parse::symbol_names_set {
     my ( $parse, $symbol, $subgrammar, $args ) = @_;
-    my $symbol_type = $subgrammar eq 'G1' ? 'G1' : 'L0';
+    my $symbol_type = $subgrammar eq 'g1' ? 'g1' : 'l0';
     my $wsyid = $parse->{next_wsyid}++;
     $parse->{symbols}->{$symbol_type}->{$symbol}->{wsyid} = $wsyid;
     for my $arg_type (keys %{$args}) {
@@ -1722,7 +1858,7 @@ sub Marpa::R3::Internal::MetaAST::Parse::prioritized_symbol {
     my $symbol_name = $base_symbol . '[' . $priority . ']';
     my $current_symbol_data =
       $parse->{symbols}
-      ->{ $Marpa::R3::Internal::SUBGRAMMAR eq 'G1' ? 'G1' : 'L0' }
+      ->{ $Marpa::R3::Internal::SUBGRAMMAR eq 'g1' ? 'g1' : 'l0' }
       ->{$symbol_name};
     return $symbol_name if defined $current_symbol_data;
 
@@ -1742,7 +1878,7 @@ sub Marpa::R3::Internal::MetaAST::Parse::discard_symbol_assign {
     my ( $parse, $symbol_name ) = @_;
 
     my $current_symbol_data =
-        $parse->{symbols}->{'L0'}->{$symbol_name};
+        $parse->{symbols}->{'l0'}->{$symbol_name};
     return $symbol_name if defined $current_symbol_data;
 
     my $symbol_data = {
@@ -1751,7 +1887,7 @@ sub Marpa::R3::Internal::MetaAST::Parse::discard_symbol_assign {
     };
     $parse->xsy_assign( $symbol_name, $symbol_data );
     $symbol_data = { xsy => $symbol_name };
-    return $parse->symbol_names_set( $symbol_name, 'L0', $symbol_data );
+    return $parse->symbol_names_set( $symbol_name, 'l0', $symbol_data );
 } ## end sub Marpa::R3::Internal::MetaAST::Parse::prioritized_symbol
 
 sub Marpa::R3::Internal::MetaAST::Parse::xsy_create {
@@ -1844,65 +1980,57 @@ sub Marpa::R3::Internal::MetaAST::xrl_create {
     return $xrlid;
 }
 
-sub Marpa::R3::Internal::MetaAST::Parse::xbnf_create {
+sub Marpa::R3::Internal::MetaAST::Parse::xpr_create {
     my ( $parse, $args, $subgrammar ) = @_;
 
     # The eXternal ALTernative is the argument hash,
     # slightly adjusted.
-    $subgrammar //= 'G1';
+    $subgrammar //= 'g1';
+    $args->{subgrammar} //= $subgrammar;
     $args->{subkey} //= 0;
-    my $rule_id = join q{,}, $args->{lhs}, @{$args->{rhs}};
-    my $hash_by_xbnfid = $parse->{xbnf}->{$subgrammar};
-    if ( exists $hash_by_xbnfid->{$rule_id} ) {
-        Marpa::R3::Internal::X->new(
-            {
-                desc       => 'Duplicate rule',
-                lhs        => $args->{lhs},
-                rhs        => $args->{rhs},
-                start      => $args->{start},
-                length     => $args->{length},
-                other_xbnf => $hash_by_xbnfid->{$rule_id},
-                to_string => sub {
-                    my $self = shift;
-                    my $pos1 = $self->{other_xbnf}->{start};
-                    my $len1 = $self->{other_xbnf}->{length};
-                    my $pos2 = $self->{start};
-                    my $len2 = $self->{length};
-
-                    my @string = ("Duplicate rules:",
-                      Marpa::R3::Internal::substr_as_2lines(
-                      'First rule',
-                      $parse->{p_dsl},
-                        $pos1, $len1, 74 ),
-                      Marpa::R3::Internal::substr_as_2lines(
-                      'Second rule',
-                      $parse->{p_dsl},
-                        $pos2, $len2, 74 )
-                        );
-
-                    push @string, q{};
-                    return join "\n", @string;
-                }
-            }
-        )->throw();
+    my $rule_id = join q{,}, $subgrammar, $args->{lhs}, @{$args->{rhs}};
+    my $hash_by_xprid = $parse->{xpr}->{$subgrammar};
+    if ( exists $hash_by_xprid->{$rule_id} ) {
+        my $other_xpr = $hash_by_xprid->{$rule_id};
+        my $pos1   = $other_xpr->{start};
+        my $len1   = $other_xpr->{length};
+        my $pos2   = $args->{start};
+        my $len2   = $args->{length};
+        my @string = (
+            "Duplicate rules:",
+            Marpa::R3::Internal::substr_as_2lines(
+                'First rule', $parse->{p_dsl}, $pos1, $len1, 74
+            ),
+            Marpa::R3::Internal::substr_as_2lines(
+                'Second rule', $parse->{p_dsl}, $pos2, $len2, 74
+            )
+        );
+        Marpa::R3::exception( join "\n", @string, q{} );
     }
-    $hash_by_xbnfid->{$rule_id} = $args;
+    $hash_by_xprid->{$rule_id} = $args;
 
     # Now create the initial working rule
-    my %wrl = ( xbnfid => $rule_id );
+    my %wrl = (
+        xprid => $rule_id,
+        subgrammar => $subgrammar,
+    );
+    # Shallow copy
     for my $field (
-        qw(lhs action rank
+        qw(lhs action priority rank
         null_ranking min separator proper )
       )
     {
         $wrl{$field} = $args->{$field} if defined $args->{$field};
     }
-    # Deep copy these arrays
-    FIELD: for my $field ( qw(rhs) ) {
-        my $xbnf_datum = $args->{$field};
-        next FIELD if not defined $xbnf_datum;
-        my @array = @{$args->{$field}};
-        $wrl{$field} = \@array;
+    # 'rhs' needs special treatment --
+    #    a deeper code and creation of the xpr_dot field
+    {
+        my $rhs = $args->{rhs};
+        my $xpr_datum = $rhs;
+        my @array = @{$rhs};
+        $wrl{rhs} = \@array;
+        $wrl{xpr_dot} = [0 .. (scalar @array) ];
+        $wrl{xpr_top} = 1;
     }
 
     # Return the initial working rule
@@ -1926,7 +2054,7 @@ sub Marpa::R3::Internal::MetaAST::Parse::internal_lexeme {
     };
     $parse->xsy_assign( $lexical_symbol, $symbol_data );
     $symbol_data = { xsy => $lexical_symbol };
-    $parse->symbol_names_set( $lexical_symbol, $_, $symbol_data ) for qw(G1 L);
+    $parse->symbol_names_set( $lexical_symbol, $_, $symbol_data ) for qw(g1 l0);
     return $lexical_symbol;
 } ## end sub Marpa::R3::Internal::MetaAST::Parse::internal_lexeme
 
