@@ -16,7 +16,7 @@ use strict;
 use warnings;
 
 use vars qw($VERSION $STRING_VERSION);
-$VERSION        = '4.001_047';
+$VERSION        = '4.001_049';
 $STRING_VERSION = $VERSION;
 ## no critic(BuiltinFunctions::ProhibitStringyEval)
 $VERSION = eval $VERSION;
@@ -37,16 +37,18 @@ our $PACKAGE = 'Marpa::R3::Scanless::R';
 # undef if there was none.
 sub Marpa::R3::Scanless::R::last_completed {
     my ( $slr, $symbol_name ) = @_;
-    my $slg  = $slr->[Marpa::R3::Internal::Scanless::R::SLG];
-    my $symbol_id       = $slg->symbol_by_name($symbol_name);
-
     my ($start, $length) = $slr->call_by_tag(
         ('@' . __FILE__ . ':' . __LINE__),
-    <<'END_OF_LUA', 'i>*', $symbol_id);
-        local recce, symbol_id  = ...
-        return recce:last_completed(symbol_id)
+    <<'END_OF_LUA', 'i', $symbol_name);
+        local slr, xsy_name  = ...
+        local xsyid = slr.slg:symbol_by_name(xsy_name)
+        if not xsyid then
+            _M.userX(
+                "last_completed(%q): no symbol with that name",
+                xsy_name)
+        end
+        return slr:last_completed(xsyid)
 END_OF_LUA
-
     return if not defined $start;
     return $start, $length;
 } ## end sub Marpa::R3::Scanless::R::last_completed
@@ -92,13 +94,6 @@ sub Marpa::R3::Internal::Scanless::meta_recce {
     return $self;
 } ## end sub Marpa::R3::Internal::Scanless::meta_recce
 
-# For error messages, make it convenient to use an SLR
-sub Marpa::R3::Scanless::R::rule_show {
-    my ( $slr, $rule_id ) = @_;
-    my $slg = $slr->[Marpa::R3::Internal::Scanless::R::SLG];
-    return $slg->rule_show($rule_id);
-}
-
 # Set those common args which are at the Perl level.
 sub perl_common_set {
     my ( $slr, $flat_args ) = @_;
@@ -116,7 +111,7 @@ sub gen_app_event_handler {
     my $event_handlers =
       $slr->[Marpa::R3::Internal::Scanless::R::EVENT_HANDLERS];
     return sub {
-        my ( $event_name, @data ) = @_;
+        my ( $event_type, $event_name, @data ) = @_;
         my $current_event =
           $slr->[Marpa::R3::Internal::Scanless::R::CURRENT_EVENT];
         if ($current_event) {
@@ -145,14 +140,28 @@ sub gen_app_event_handler {
         }
         $slr->[Marpa::R3::Internal::Scanless::R::CURRENT_EVENT] = $event_name;
         my $retour = $handler->( $slr, $event_name, @data ) // 'ok';
-        $slr->[Marpa::R3::Internal::Scanless::R::CURRENT_EVENT] = undef;
-        if ( $retour ne 'ok' and $retour ne 'pause' ) {
+
+        RETOUR_CHECK: {
+            if ($retour eq 'ok') {
+                if ($event_type eq 'before lexeme') {
+                    Marpa::R3::exception(
+                        qq{Bad return from event handler for event "$event_name"\n},
+                        qq{  Event type was "$event_type"\n},
+                        qq{  Return from handler was "$retour"\n},
+                        qq{  A handler of type "$event_type" must return "pause"\n},
+                    );
+                }
+                last RETOUR_CHECK;
+            }
+            last RETOUR_CHECK if $retour eq 'pause';
             Marpa::R3::exception(
                 qq{Bad return from event handler for event "$event_name"\n},
+                qq{  Event type was "$event_type"\n},
                 qq{  Return from handler was "$retour"\n},
                 qq{  Handler must return "ok" or "pause"\n},
             );
         }
+        $slr->[Marpa::R3::Internal::Scanless::R::CURRENT_EVENT] = undef;
         return 'ok', $retour;
     };
 }
@@ -235,7 +244,7 @@ END_OF_LUA
 
     $slr->[Marpa::R3::Internal::Scanless::R::REGIX]  = $regix;
 
-    my ( $events ) = $slr->coro_by_tag(
+    $slr->coro_by_tag(
         ( '@' . __FILE__ . ':' . __LINE__ ),
         {
             signature => 's',
@@ -252,13 +261,11 @@ END_OF_LUA
         <<'END_OF_LUA');
         local slr, flat_args = ...
         _M.wrap(function ()
-            local _, events = glue.convert_libmarpa_events(slr)
-            return 'ok', events
+            slr:convert_libmarpa_events(slr)
+            return 'ok'
         end)
 END_OF_LUA
 
-    $slr->[Marpa::R3::Internal::Scanless::R::EVENTS] = $events;
-
     return $slr;
 } ## end sub Marpa::R3::Scanless::R::new
 
@@ -284,8 +291,7 @@ sub Marpa::R3::Scanless::R::DESTROY {
     local slr = ...
     slr:valuation_reset()
     local regix = slr.regix
-    local registry = debug.getregistry()
-    _M.unregister(registry, regix)
+    _M.unregister(_M.registry, regix)
 END_OF_LUA
 }
 
@@ -345,11 +351,6 @@ sub Marpa::R3::Scanless::R::read {
             return 'ok'
 END_OF_LUA
 
-    my $event_count =
-      scalar @{ $slr->[Marpa::R3::Internal::Scanless::R::EVENTS] };
-
-    return 0 if $event_count > 0;
-
     return $slr->resume( $start_pos, $length );
 
 } ## end sub Marpa::R3::Scanless::R::read
@@ -368,7 +369,7 @@ sub Marpa::R3::Scanless::R::resume {
     my $trace_file_handle =
       $slr->[Marpa::R3::Internal::Scanless::R::TRACE_FILE_HANDLE];
 
-    my ($events) = $slr->coro_by_tag(
+    $slr->coro_by_tag(
         ( '@' . __FILE__ . ':' . __LINE__ ),
         {
             signature => 'ii',
@@ -420,18 +421,9 @@ sub Marpa::R3::Scanless::R::resume {
       )
 END_OF_LUA
 
-    $slr->[Marpa::R3::Internal::Scanless::R::EVENTS] = $events;
-
     return $slr->pos();
 } ## end sub Marpa::R3::Scanless::R::resume
 
-sub Marpa::R3::Scanless::R::events {
-    my ($slr) = @_;
-    my $events = $slr->[Marpa::R3::Internal::Scanless::R::EVENTS];
-    # say Data::Dumper::Dumper($events);
-    return $events // [];
-}
-
 sub character_describe {
     my ($slr, $codepoint) = @_;
 
@@ -581,7 +573,7 @@ sub lc_brief {
     <<'END_OF_LUA', 'ii', $pos, ($block // -1));
         local slr, pos, block = ...
         if block < 0 then block = nil end
-        return slr:lc_brief(pos, block)
+        return slr:lc_brief(block, pos)
 END_OF_LUA
      return $result;
 }
@@ -599,155 +591,71 @@ END_OF_LUA
 
 }
 
-sub Marpa::R3::Scanless::R::show_progress {
+sub Marpa::R3::Scanless::R::progress_show {
     my ( $slr, $start_ordinal, $end_ordinal ) = @_;
-    my $slg = $slr->[Marpa::R3::Internal::Scanless::R::SLG];
-
-    my ($last_ordinal) = $slr->call_by_tag( ( '@' . __FILE__ . ':' . __LINE__ ),
-        'local recce = ...; return recce.g1:latest_earley_set()', '' );
-
-    if ( not defined $start_ordinal ) {
-        $start_ordinal = $last_ordinal;
-    }
-    if ( $start_ordinal < 0 ) {
-        $start_ordinal += $last_ordinal + 1;
-    }
-    else {
-        if ( $start_ordinal < 0 or $start_ordinal > $last_ordinal ) {
-            return
-"Marpa::R3::Scanless::R::show_progress start index is $start_ordinal, "
-              . "must be in range 0-$last_ordinal";
-        }
-    } ## end else [ if ( $start_ordinal < 0 ) ]
-
-    if ( not defined $end_ordinal ) {
-        $end_ordinal = $start_ordinal;
-    }
-    else {
-        my $end_ordinal_argument = $end_ordinal;
-        if ( $end_ordinal < 0 ) {
-            $end_ordinal += $last_ordinal + 1;
-        }
-        if ( $end_ordinal < 0 ) {
-            return
-"Marpa::R3::Scanless::R::show_progress end index is $end_ordinal_argument, "
-              . sprintf ' must be in range %d-%d', -( $last_ordinal + 1 ),
-              $last_ordinal;
-        } ## end if ( $end_ordinal < 0 )
-    } ## end else [ if ( not defined $end_ordinal ) ]
-
-    my $text = q{};
-    for my $current_ordinal ( $start_ordinal .. $end_ordinal ) {
-        my $current_earleme     = $slr->earleme($current_ordinal);
-        my %by_rule_by_position = ();
-        for my $progress_item ( @{ $slr->progress($current_ordinal) } ) {
-            my ( $rule_id, $position, $origin ) = @{$progress_item};
-            if ( $position < 0 ) {
-                ($position) = $slg->call_by_tag(
-                    ( '@' . __FILE__ . ':' . __LINE__ ),
-'local grammar, rule_id = ...; return grammar.g1:rule_length(rule_id)',
-                    'i',
-                    $rule_id
-                );
-            }
-            $by_rule_by_position{$rule_id}->{$position}->{$origin}++;
-        } ## end for my $progress_item ( @{ $recce->progress($current_ordinal...)})
-
-        for my $rule_id ( sort { $a <=> $b } keys %by_rule_by_position ) {
-            my $by_position = $by_rule_by_position{$rule_id};
-            for my $position ( sort { $a <=> $b } keys %{$by_position} ) {
-                my $raw_origins   = $by_position->{$position};
-                my @origins       = sort { $a <=> $b } keys %{$raw_origins};
-                my $origins_count = scalar @origins;
-                my $origin_desc;
-                if ( $origins_count <= 3 ) {
-                    $origin_desc = join q{,}, @origins;
-                }
-                else {
-                    $origin_desc = $origins[0] . q{...} . $origins[-1];
-                }
-
-                my ($rhs_length) = $slg->call_by_tag(
-                    ( '@' . __FILE__ . ':' . __LINE__ ),
-'local grammar, rule_id = ...; return grammar.g1:rule_length(rule_id)',
-                    'i',
-                    $rule_id
-                );
-                my @item_text;
-
-                my $dotted_type = "R";
-                if ( $position >= $rhs_length ) {
-                    $dotted_type = "F";
-                    push @item_text, "F$rule_id";
-                }
-                elsif ($position) {
-                    push @item_text, "R$rule_id:$position";
-                }
-                else {
-                    $dotted_type = "P";
-                    push @item_text, "P$rule_id";
-                }
-                push @item_text, "x$origins_count" if $origins_count > 1;
-                push @item_text, q{@} . $origin_desc . q{-} . $current_earleme;
-
-                # For origins[0], we apply
-                # -1 to convert earley set to G1, then
-                # +1 one because it is an origin and the character
-                # don't begin until the next Earley set
-                # -- in other words, they balance and we do nothing
-
-                my ($input_range) =
-                  $slr->call_by_tag( ( '@' . __FILE__ . ':' . __LINE__ ),
-                    <<'END_OF_LUA', 'sii', $dotted_type, ($origins[0]+0), $current_ordinal );
-    local slr, dotted_type, g1_first, current_ordinal = ...
-    if current_ordinal <= 0 then return 'B0L0c0' end
-    if dotted_type == 'P' then
-        local block, pos = slr:g1_pos_to_l0_first(current_ordinal)
-        return slr:lc_brief(pos, block)
-    end
-    if g1_first < 0 then g1_first = 0 end
-    local g1_last = current_ordinal - 1
-    local l0_first_b, l0_first_p = slr:g1_pos_to_l0_first(g1_first)
-    local l0_last_b, l0_last_p = slr:g1_pos_to_l0_last(g1_last)
-    return slr:lc_range_brief(l0_first_b, l0_first_p, l0_last_b, l0_last_p)
+    my ($text) = $slr->call_by_tag(
+            ( '@' . __FILE__ . ':' . __LINE__ ),
+            <<'END_OF_LUA', 'ii', $start_ordinal, $end_ordinal );
+    local slr, start_ordinal_arg, end_ordinal_arg = ...
+    return slr:progress_show(start_ordinal_arg, end_ordinal_arg )
 END_OF_LUA
-
-                push @item_text, $input_range;
-
-                push @item_text, $slg->show_dotted_rule( $rule_id, $position );
-                $text .= ( join q{ }, @item_text ) . "\n";
-            } ## end for my $position ( sort { $a <=> $b } keys %{...})
-        } ## end for my $rule_id ( sort { $a <=> $b } keys ...)
-
-    } ## end for my $current_ordinal ( $start_ordinal .. $end_ordinal)
     return $text;
 }
 
 sub Marpa::R3::Scanless::R::progress {
     my ( $slr, $ordinal_arg ) = @_;
-
     my ($result) = $slr->call_by_tag(
         ('@' . __FILE__ . ':' . __LINE__),
     <<'END_OF_LUA', 'i>0', ($ordinal_arg // -1));
-    local recce, ordinal_arg = ...
-    return recce:progress(ordinal_arg)
+    local slr, ordinal_arg = ...
+    return slr:progress(ordinal_arg)
 END_OF_LUA
+    return $result;
+}
 
+sub Marpa::R3::Scanless::R::g1_progress_show {
+    my ( $slr, $start_ordinal, $end_ordinal ) = @_;
+    my ($text) = $slr->call_by_tag(
+            ( '@' . __FILE__ . ':' . __LINE__ ),
+            <<'END_OF_LUA', 'ii', $start_ordinal, $end_ordinal );
+    local slr, start_ordinal_arg, end_ordinal_arg = ...
+    return slr:g1_progress_show(start_ordinal_arg, end_ordinal_arg )
+END_OF_LUA
+    return $text;
+}
+
+sub Marpa::R3::Scanless::R::g1_progress {
+    my ( $slr, $ordinal_arg ) = @_;
+    my ($result) = $slr->call_by_tag(
+        ('@' . __FILE__ . ':' . __LINE__),
+    <<'END_OF_LUA', 'i>0', ($ordinal_arg // -1));
+    local slr, ordinal_arg = ...
+    return slr:g1_progress(ordinal_arg)
+END_OF_LUA
     return $result;
 }
 
 sub Marpa::R3::Scanless::R::terminals_expected {
     my ($slr)      = @_;
-    my $slg        = $slr->[Marpa::R3::Internal::Scanless::R::SLG];
-    my ($terminals_expected) = $slr->call_by_tag(
+    my ($results) = $slr->call_by_tag(
     ('@' . __FILE__ . ':' . __LINE__),
-    <<'END_OF_LUA', '>0');
-    local recce = ...
-    local terminals_expected = recce.g1:terminals_expected()
-    return terminals_expected
+    <<'END_OF_LUA', '');
+    local slr = ...
+    local slg = slr.slg
+    local g1g = slg.g1
+    local terminals_expected = slr.g1:terminals_expected()
+    local results = {}
+    for ix = 1, #terminals_expected do
+        local g1_symbol_id = terminals_expected[ix]
+        local xsy = g1g:_xsy(g1_symbol_id)
+        if xsy then
+            results[#results+1] = xsy.name
+        end
+    end
+    return results
 END_OF_LUA
 
-    return [ map { $slg->g1_symbol_name($_) } @{$terminals_expected} ];
+    return $results;
 }
 
 sub Marpa::R3::Scanless::R::exhausted {
@@ -815,28 +723,43 @@ END_OF_LUA
 sub Marpa::R3::Scanless::R::lexeme_alternative {
     my ( $slr, $symbol_name, @value ) = @_;
 
+    if (Scalar::Util::tainted($value[1])) {
+        Marpa::R3::exception(
+              "Problem in Marpa::R3: Attempt to use a tainted token value\n",
+              "Marpa::R3 is insecure for use with tainted data\n");
+    }
+
     Marpa::R3::exception(
         "slr->alternative(): symbol name is undefined\n",
         "    The symbol name cannot be undefined\n"
     ) if not defined $symbol_name;
 
     my $slg        = $slr->[Marpa::R3::Internal::Scanless::R::SLG];
-    my $symbol_id  = $slg->symbol_by_name($symbol_name);
-    if ( not defined $symbol_id ) {
-        Marpa::R3::exception(
-            qq{slr->alternative(): symbol "$symbol_name" does not exist});
-    }
+    my ($g1_token_id) = 
+             $slr->call_by_tag( ( '@' . __FILE__ . ':' . __LINE__ ),
+                <<'END_OF_LUA', 's', $symbol_name );
+        local slr, symbol_name = ...
+        local slg = slr.slg
+        local xsy = slg.xsys[symbol_name]
+        if not xsy then
+            _M.userX(
+                "slr->lexeme_alternative(): symbol %q does not exist",
+                symbol_name)
+        end
+        local lexeme = xsy.lexeme
+        if not lexeme then
+            _M.userX(
+                "slr->lexeme_alternative(): symbol %q is not a lexeme",
+                symbol_name)
+        end
+        return lexeme.g1_isy.id
+END_OF_LUA
 
-    if (Scalar::Util::tainted($value[1])) {
-        Marpa::R3::exception(
-              "Problem in Marpa::R3: Attempt to use a tainted token value\n",
-              "Marpa::R3 is insecure for use with tainted data\n");
-    }
     my $result;
   DO_ALTERNATIVE: {
         if ( scalar @value == 0 ) {
             ($result) = $slr->call_by_tag( ( '@' . __FILE__ . ':' . __LINE__ ),
-                <<'END_OF_LUA', 'i', $symbol_id );
+                <<'END_OF_LUA', 'i', $g1_token_id );
         local slr, symbol_id = ...
         local token_ix = _M.defines.TOKEN_VALUE_IS_LITERAL
         local g1r = slr.g1
@@ -849,7 +772,7 @@ END_OF_LUA
         my $value = $value[0];
         if ( not defined $value ) {
             ($result) = $slr->call_by_tag( ( '@' . __FILE__ . ':' . __LINE__ ),
-                <<'END_OF_LUA', 'i', $symbol_id );
+                <<'END_OF_LUA', 'i', $g1_token_id );
         local slr, symbol_id = ...
         local token_ix = _M.defines.TOKEN_VALUE_IS_UNDEF
         local g1r = slr.g1
@@ -860,7 +783,7 @@ END_OF_LUA
             last DO_ALTERNATIVE;
         }
             ($result) = $slr->call_by_tag( ( '@' . __FILE__ . ':' . __LINE__ ),
-                <<'END_OF_LUA', 'iS', $symbol_id, $value );
+                <<'END_OF_LUA', 'iS', $g1_token_id, $value );
         local slr, symbol_id, token_sv = ...
         local token_ix = #slr.token_values + 1
         slr.token_values[token_ix] = token_sv
@@ -902,7 +825,7 @@ sub Marpa::R3::Scanless::R::lexeme_complete {
     my $trace_file_handle =
         $slr->[Marpa::R3::Internal::Scanless::R::TRACE_FILE_HANDLE];
 
-    my ($return_value, $events) = $slr->coro_by_tag(
+    my ($return_value) = $slr->coro_by_tag(
         ( '@' . __FILE__ . ':' . __LINE__ ),
         {
            signature => 'ii',
@@ -938,14 +861,12 @@ sub Marpa::R3::Scanless::R::lexeme_complete {
           slg.g1.error()
       end
       _M.wrap(function ()
-          local _, events = glue.convert_libmarpa_events(slr)
-          return 'ok', complete_val, events
+          slr:convert_libmarpa_events(slr)
+          return 'ok', complete_val
       end
       )
 END_OF_LUA
 
-    $slr->[Marpa::R3::Internal::Scanless::R::EVENTS] = $events;
-
     return $return_value;
 
 } ## end sub Marpa::R3::Scanless::R::lexeme_complete
@@ -989,6 +910,12 @@ sub Marpa::R3::Scanless::R::g1_to_l0_first {
     return $slr->call_by_tag( ( '@' . __FILE__ . ':' . __LINE__ ),
         <<'END_OF_LUA', 'i', $g1_pos  );
         local slr, g1_pos = ...
+        g1_pos = math.tointeger(g1_pos)
+        if not g1_pos then
+            _M.userX(
+                "g1_to_l0_first(%s): argument must be an integer",
+                g1_pos)
+        end
         return slr:g1_pos_to_l0_first(g1_pos)
 END_OF_LUA
 }
@@ -999,6 +926,12 @@ sub Marpa::R3::Scanless::R::g1_to_l0_last {
     return $slr->call_by_tag( ( '@' . __FILE__ . ':' . __LINE__ ),
         <<'END_OF_LUA', 'i', $g1_pos  );
         local slr, g1_pos = ...
+        g1_pos = math.tointeger(g1_pos)
+        if not g1_pos then
+            _M.userX(
+                "g1_to_l0_last(%s): argument must be an integer",
+                g1_pos)
+        end
         return slr:g1_pos_to_l0_last(g1_pos)
 END_OF_LUA
 }
@@ -1264,25 +1197,25 @@ sub Marpa::R3::Scanless::R::lexeme_priority_set {
         local g1g = slg.g1
         local lexeme_id = g1g.isyid_by_name[lexeme_name]
         if not lexeme_id then
-            _M.userX(string.format(
+            _M.userX(
                 "lexeme_priority_set(): no such symbol as %q",
                 lexeme_name
-            ))
+            )
         end
         if type(new_priority) ~= 'number' then
-            _M.userX(string.format(
+            _M.userX(
                 "lexeme_priority_set(): priority is not a number, it is %s",
                 new_priority
-            ))
+            )
         end
         local g_lexeme_data = slg.g1.isys[lexeme_id]
         local r_lexeme_data = recce.g1_isys[lexeme_id]
-        if not g_lexeme_data.is_lexeme then
+        if not g_lexeme_data.lexeme then
             print(inspect(lexeme_data))
-            _M.userX(string.format(
+            _M.userX(
                 "lexeme_priority_set(): %q is not a lexeme",
                 lexeme_name
-            ))
+            )
         end
         local old_priority = r_lexeme_data.lexeme_priority
         r_lexeme_data.lexeme_priority = new_priority
@@ -1381,7 +1314,7 @@ END_OF_LUA
 }
 
 # not to be documented
-sub Marpa::R3::Scanless::R::show_earley_sets {
+sub Marpa::R3::Scanless::R::earley_sets_show {
     my ($slr)                = @_;
 
     my ($last_completed_earleme, $furthest_earleme) = $slr->call_by_tag(
@@ -1396,7 +1329,7 @@ END_OF_LUA
         . "Furthest: $furthest_earleme\n";
     LIST: for ( my $ix = 0;; $ix++ ) {
         my $set_desc =
-          $slr->Marpa::R3::Scanless::R::show_earley_set( $ix );
+          $slr->Marpa::R3::Scanless::R::earley_set_show( $ix );
         last LIST if not $set_desc;
         $text .= "Earley Set $ix\n$set_desc";
     }
@@ -1404,7 +1337,7 @@ END_OF_LUA
 }
 
 # not to be documented
-sub Marpa::R3::Scanless::R::show_earley_set {
+sub Marpa::R3::Scanless::R::earley_set_show {
     my ( $slr, $traced_set_id ) = @_;
     my $slg     = $slr->[Marpa::R3::Internal::Scanless::R::SLG];
 
@@ -1430,14 +1363,14 @@ END_OF_LUA
 
         my %item_data = @{$item_data};
 
-        my $irl_id       = $item_data{irl_id};
+        my $nrl_id       = $item_data{nrl_id};
         my $dot_position = $item_data{dot_position};
         my $ahm_desc;
         if ( $dot_position < 0 ) {
-            $ahm_desc = sprintf( 'R%d$', $irl_id );
+            $ahm_desc = sprintf( 'R%d$', $nrl_id );
         }
         else {
-            $ahm_desc = sprintf( 'R%d:%d', $irl_id, $dot_position );
+            $ahm_desc = sprintf( 'R%d:%d', $nrl_id, $dot_position );
         }
         my $ahm_id_of_yim  = $item_data{ahm_id_of_yim};
         my $origin_earleme = $item_data{origin_earleme};
@@ -1450,7 +1383,7 @@ END_OF_LUA
         push @lines,
             qq{  }
           . $ahm_desc . q{: }
-          . $slg->show_dotted_irl( $irl_id, $dot_position );
+          . $slg->dotted_nrl_show( $nrl_id, $dot_position );
 
         push @sorted_data, @lines;
 
@@ -1478,7 +1411,7 @@ END_OF_LUA
 
                 my @pieces = ();
                 if ( defined $predecessor_ahm ) {
-                    my $ahm_desc = $slg->show_briefer_ahm($predecessor_ahm);
+                    my $ahm_desc = $slg->briefer_ahm($predecessor_ahm);
                     push @pieces,
                         'c='
                       . $ahm_desc . q{@}
@@ -1530,12 +1463,12 @@ END_OF_LUA
                 my $ahm_id         = $completion_link_data{ahm_id};
                 my $origin_earleme = $completion_link_data{origin_earleme};
                 my $middle_earleme = $completion_link_data{middle_earleme};
-                my $ahm_desc       = $slg->show_briefer_ahm($ahm_id);
+                my $ahm_desc       = $slg->briefer_ahm($ahm_id);
 
                 my @pieces = ();
                 if ( defined $predecessor_ahm_id ) {
                     my $predecessor_ahm_desc =
-                      $slg->show_briefer_ahm($predecessor_ahm_id);
+                      $slg->briefer_ahm($predecessor_ahm_id);
                     push @pieces,
                         'p='
                       . $predecessor_ahm_desc . '@'
@@ -1580,7 +1513,7 @@ END_OF_LUA
                 my $leo_transition_symbol =
                   $leo_link_data{leo_transition_symbol};
                 my $ahm_id   = $leo_link_data{ahm_id};
-                my $ahm_desc = $slg->show_briefer_ahm($ahm_id);
+                my $ahm_desc = $slg->briefer_ahm($ahm_id);
 
                 my @pieces = ();
                 push @pieces,
@@ -1650,38 +1583,38 @@ END_OF_LUA
 }
 
 # not to be documented
-sub Marpa::R3::Scanless::R::show_or_nodes {
+sub Marpa::R3::Scanless::R::or_nodes_show {
     my ( $slr ) = @_;
 
     my ($result) = $slr->call_by_tag(
     ('@' . __FILE__ . ':' . __LINE__),
     <<'END_OF_LUA', '');
     local recce = ...
-    return recce:show_or_nodes()
+    return recce:or_nodes_show()
 END_OF_LUA
 
     return $result;
 }
 
 # not to be documented
-sub Marpa::R3::Scanless::R::show_and_nodes {
+sub Marpa::R3::Scanless::R::and_nodes_show {
     my ( $slr ) = @_;
     my ($result) = $slr->call_by_tag(
     ('@' . __FILE__ . ':' . __LINE__),
     <<'END_OF_LUA', '');
     local recce = ...
-    return recce:show_and_nodes()
+    return recce:and_nodes_show()
 END_OF_LUA
 
     return $result;
 }
 
 # not to be documented
-sub Marpa::R3::Scanless::R::show_tree {
+sub Marpa::R3::Scanless::R::tree_show {
     my ( $slr, $verbose ) = @_;
     my $text = q{};
     NOOK: for ( my $nook_id = 0; 1; $nook_id++ ) {
-        my $nook_text = $slr->show_nook( $nook_id, $verbose );
+        my $nook_text = $slr->nook_show( $nook_id, $verbose );
         last NOOK if not defined $nook_text;
         $text .= "$nook_id: $nook_text";
     }
@@ -1689,7 +1622,7 @@ sub Marpa::R3::Scanless::R::show_tree {
 }
 
 # not to be documented
-sub Marpa::R3::Scanless::R::show_nook {
+sub Marpa::R3::Scanless::R::nook_show {
     my ( $slr, $nook_id, $verbose ) = @_;
 
     my ($or_node_id, $text) = $slr->call_by_tag(
@@ -1766,14 +1699,14 @@ END_OF_LUA
 }
 
 # not to be documented
-sub Marpa::R3::Scanless::R::show_bocage {
+sub Marpa::R3::Scanless::R::bocage_show {
     my ($slr)     = @_;
 
     my ($result) = $slr->call_by_tag(
     ('@' . __FILE__ . ':' . __LINE__),
         <<'END_OF_LUA', '');
         local recce = ...
-        return recce:show_bocage()
+        return recce:bocage_show()
 END_OF_LUA
 
     return $result;
@@ -1782,7 +1715,7 @@ END_OF_LUA
 # not to be documented
 sub Marpa::R3::Scanless::R::verbose_or_node {
     my ( $slr, $or_node_id ) = @_;
-    my ($text, $irl_id, $position)
+    my ($text, $nrl_id, $position)
         = $slr->call_by_tag(
     ('@' . __FILE__ . ':' . __LINE__),
         <<'END_OF_LUA', 'i', $or_node_id);
@@ -1791,7 +1724,6 @@ sub Marpa::R3::Scanless::R::verbose_or_node {
         local origin = bocage:_or_node_origin(or_node_id)
         if not origin then return end
         local set = bocage:_or_node_set(or_node_id)
-        local irl_id = bocage:_or_node_irl(or_node_id)
         local position = bocage:_or_node_position(or_node_id)
         local g1r = recce.g1
         local origin_earleme = g1r:earleme(origin)
@@ -1809,7 +1741,7 @@ END_OF_LUA
 
     my $slg = $slr->[Marpa::R3::Internal::Scanless::R::SLG];
     $text .= ( q{ } x 4 )
-        . $slg->show_dotted_irl( $irl_id, $position ) . "\n";
+        . $slg->dotted_nrl_show( $nrl_id, $position ) . "\n";
     return $text;
 }
 
