@@ -16,7 +16,7 @@ use warnings;
 use strict;
 
 use vars qw($VERSION $STRING_VERSION);
-$VERSION        = '4.001_047';
+$VERSION        = '4.001_049';
 $STRING_VERSION = $VERSION;
 ## no critic (BuiltinFunctions::ProhibitStringyEval)
 $VERSION = eval $VERSION;
@@ -172,6 +172,20 @@ END_OF_LUA
     return $start, $end;
 } ## end sub Marpa::R3::Context::g1_range
 
+sub Marpa::R3::Context::lc_range {
+    my $slr = $Marpa::R3::Context::slr;
+    my ( $lc_range ) =
+      $slr->call_by_tag( ( '@' . __FILE__ . ':' . __LINE__ ), <<'END_OF_LUA', '>*' );
+local slr = ...
+local g1_first = slr.this_step.start_es_id
+local g1_last = slr.this_step.es_id - 1
+local l0_first_b, l0_first_p = slr:g1_pos_to_l0_first(g1_first)
+local l0_last_b, l0_last_p = slr:g1_pos_to_l0_last(g1_last)
+return slr:lc_range_brief(l0_first_b, l0_first_p, l0_last_b, l0_last_p)
+END_OF_LUA
+    return $lc_range;
+}
+
 sub Marpa::R3::Context::g1_span {
     my $slr = $Marpa::R3::Context::slr;
     my ( $start, $length ) =
@@ -341,7 +355,7 @@ END_OF_LUA
     my $resolution = resolve_action( $slg, $action_name, \$resolve_error );
 
     if ( not $resolution ) {
-        my $rule_desc = $slg->rule_show($irlid);
+        my $rule_desc = $slg->g1_rule_show($irlid);
         Marpa::R3::exception(
             "Could not resolve rule action named '$action_name'\n",
             "  Rule was $rule_desc\n",
@@ -380,9 +394,9 @@ sub rule_blessing_find {
         local slg, irlid, bless_package = ...
         local irl = slg.g1.irls[irlid]
         local blessing = '::undef'
-        local xbnf = irl.xbnf
-        if xbnf then
-            blessing = xbnf.bless or '::undef'
+        local xpr = irl.xpr
+        if xpr then
+            blessing = xpr.bless or '::undef'
         end
         if blessing == '::undef' then return blessing end
         if #bless_package == 0 then
@@ -418,13 +432,13 @@ sub resolve_grammar {
 
     my $rule_resolutions = [];
 
-  RULE: for my $irlid ( $slg->rule_ids() ) {
+  RULE_ID: for (my $iter = $slg->g1_rule_ids_gen(); defined ( my $irlid = $iter->());) {
 
         my $rule_resolution = resolve_rule_by_id( $slg, $irlid );
         $rule_resolution //= $default_action_resolution;
 
         if ( not $rule_resolution ) {
-            my $rule_desc = $slg->rule_show($irlid);
+            my $rule_desc = $slg->g1_rule_show($irlid);
 
             my ($action) =
               $slg->call_by_tag( ( '@' . __FILE__ . ':' . __LINE__ ),
@@ -458,7 +472,7 @@ END_OF_LUA
 qq{Attempt to bless, but improper semantics: "$semantics"\n},
                         qq{  Blessing: "$blessing"\n},
                         '  Rule: ',
-                        $slg->brief_rule($irlid)
+                        $slg->g1_rule_show($irlid)
                     );
                 } ## end CHECK_SEMANTICS:
             } ## end if ( $blessing ne '::undef' )
@@ -469,7 +483,7 @@ qq{Attempt to bless, but improper semantics: "$semantics"\n},
 
         $rule_resolutions->[$irlid] = $rule_resolution;
 
-    } ## end RULE: for my $rule_id ( $slg->rule_ids() )
+    }
 
     if ( $trace_actions >= 2 ) {
 
@@ -485,7 +499,7 @@ END_OF_LUA
             my ( $resolution_name, $closure ) =
               @{ $rule_resolutions->[$rule_id] };
             say {$trace_file_handle} 'Rule ',
-              $slg->brief_rule($rule_id),
+              $slg->g1_rule_show($rule_id),
               qq{ resolves to "$resolution_name"}
               or Marpa::R3::exception('print to trace handle failed');
         }
@@ -508,7 +522,7 @@ END_OF_LUA
             my $message =
                 "Could not determine lexeme's semantics\n"
               . q{  Lexeme was }
-              . $slg->g1_symbol_name($lexeme_id) . "\n";
+              . $slg->g1_symbol_display($lexeme_id) . "\n";
             Marpa::R3::exception($message);
         } ## end if ( not defined $semantics )
         my $blessing = lexeme_blessing_find( $slg, $lexeme_id );
@@ -516,7 +530,7 @@ END_OF_LUA
             my $message =
                 "Could not determine lexeme's blessing\n"
               . q{  Lexeme was }
-              . $slg->g1_symbol_name($lexeme_id) . "\n";
+              . $slg->g1_symbol_display($lexeme_id) . "\n";
             Marpa::R3::exception($message);
         } ## end if ( not defined $blessing )
         $lexeme_resolutions[$lexeme_id] = [ $semantics, $blessing ];
@@ -634,10 +648,10 @@ END_OF_LUA
 }
 
 # For diagnostics
-sub brief_rule_list {
+sub g1_show_rule_list {
     my ( $slg, $rule_ids ) = @_;
-    my @brief_rules = map { $slg->brief_rule($_) } @{$rule_ids};
-    return join q{}, map { q{    } . $_ . "\n" } @brief_rules;
+    my @rules = map { $slg->g1_rule_show($_) } @{$rule_ids};
+    return join q{}, map { q{    } . $_ . "\n" } @rules;
 }
 
 sub registrations_set {
@@ -655,7 +669,7 @@ sub registrations_set {
             if ( $type eq 'nulling' ) {
                 say {$trace_file_handle}
                   "Registering semantics for nulling symbol: ",
-                  $slg->g1_symbol_name($id),
+                  $slg->g1_symbol_display($id),
                   "\n", '  Semantics are ', $slg->show_semantics(@raw_ops)
                   or Marpa::R3::exception('Cannot say to trace file handle');
                 last PRINT_TRACES;
@@ -663,7 +677,7 @@ sub registrations_set {
             if ( $type eq 'rule' ) {
                 say {$trace_file_handle}
                   "Registering semantics for $type: ",
-                  $slg->brief_rule($id),
+                  $slg->g1_rule_show($id),
                   '  Semantics are ', $slg->show_semantics(@raw_ops)
                   or Marpa::R3::exception('Cannot say to trace file handle');
                 last PRINT_TRACES;
@@ -671,7 +685,7 @@ sub registrations_set {
             if ( $type eq 'token' ) {
                 say {$trace_file_handle}
                   "Registering semantics for $type: ",
-                  $slg->g1_symbol_name($id),
+                  $slg->g1_symbol_display($id),
                   "\n", '  Semantics are ', $slg->show_semantics(@raw_ops)
                   or Marpa::R3::exception('Cannot say to trace file handle');
                 last PRINT_TRACES;
@@ -731,8 +745,7 @@ sub registrations_find {
     # Set the arrays, and perform various checks on the resolutions
     # we received
     {
-      RULE:
-        for my $irlid ( $slg->rule_ids() ) {
+      RULE: for (my $iter = $slg->g1_rule_ids_gen(); defined ( my $irlid = $iter->());) {
             my ( $new_resolution, $closure, $semantics, $blessing ) =
               @{ $rule_resolutions->[$irlid] };
             my ($lhs_id) =
@@ -766,7 +779,7 @@ END_OF_LUA
 
                 Marpa::R3::exception(
                     q{Unknown semantics for rule },
-                    $slg->brief_rule($irlid),
+                    $slg->g1_rule_show($irlid),
                     "\n",
                     qq{    Semantics were specified as "$semantics"\n}
                 );
@@ -790,7 +803,7 @@ qq{Fatal error: Attempt to bless a rule that resolves to a scalar constant\n},
                             Data::Dumper::Dumper($closure),
                             qq{  Blessing is "$blessing"\n},
                             q{  Rule is: },
-                            $slg->brief_rule($irlid),
+                            $slg->g1_rule_show($irlid),
                             "\n",
 qq{  Cannot bless rule when it resolves to a scalar constant},
                             "\n",
@@ -803,7 +816,7 @@ qq{  Cannot bless rule when it resolves to a scalar constant},
                 Marpa::R3::exception(
                     qq{Cannot bless rule when the semantics are "$semantics"},
                     q{  Rule is: },
-                    $slg->brief_rule($irlid),
+                    $slg->g1_rule_show($irlid),
                     "\n",
                     qq{  Blessing is "$blessing"\n},
                     qq{  Semantics are "$semantics"\n}
@@ -818,7 +831,7 @@ qq{  Cannot bless rule when it resolves to a scalar constant},
     # and that means more than one semantics might be specified for
     # the nullable symbol.  This logic deals with that.
     my @nullable_rule_ids_by_lhs = ();
-  RULE: for my $irlid ( $slg->rule_ids() ) {
+      RULE: for (my $iter = $slg->g1_rule_ids_gen(); defined ( my $irlid = $iter->());) {
 
         my ( $lhs_id, $rule_is_nullable ) =
           $slg->call_by_tag( ( '@' . __FILE__ . ':' . __LINE__ ),
@@ -851,11 +864,11 @@ END_OF_LUA
             my ( $resolution_name, $closure ) =
               @{ $rule_resolutions->[$resolution_rule] };
             if ($trace_actions) {
-                my $lhs_name = $slg->g1_symbol_name($lhs_id);
+                my $lhs_name = $slg->g1_symbol_display($lhs_id);
                 say {$trace_file_handle}
                   qq{Nulled symbol "$lhs_name" },
                   qq{ resolved to "$resolution_name" from rule },
-                  $slg->brief_rule($resolution_rule)
+                  $slg->g1_rule_show($resolution_rule)
                   or Marpa::R3::exception('print to trace handle failed');
             } ## end if ($trace_actions)
             $null_symbol_closures[$lhs_id] = $resolution_rule;
@@ -885,11 +898,11 @@ END_OF_LUA
             my ( $resolution_name, $closure ) =
               @{ $rule_resolutions->[$resolution_rule] };
             if ($trace_actions) {
-                my $lhs_name = $slg->g1_symbol_name($lhs_id);
+                my $lhs_name = $slg->g1_symbol_display($lhs_id);
                 say {$trace_file_handle}
                   qq{Nulled symbol "$lhs_name" },
                   qq{ resolved to "$resolution_name" from rule },
-                  $slg->brief_rule($resolution_rule)
+                  $slg->g1_rule_show($resolution_rule)
                   or Marpa::R3::exception('print to trace handle failed');
             } ## end if ($trace_actions)
             $null_symbol_closures[$lhs_id] = $resolution_rule;
@@ -914,11 +927,11 @@ END_OF_LUA
             {
                 Marpa::R3::exception(
                     'When nulled, symbol ',
-                    $slg->g1_symbol_name($lhs_id),
+                    $slg->g1_symbol_display($lhs_id),
                     qq{  can have more than one semantics\n},
                     qq{  Marpa needs there to be only one semantics\n},
                     qq{  The rules involved are:\n},
-                    brief_rule_list( $slg, $irlids )
+                    g1_show_rule_list( $slg, $irlids )
                 );
             } ## end if ( $first_closure_name ne $other_closure_name or ...)
         } ## end OTHER_RESOLUTION: for my $other_resolution (@other_resolutions)
@@ -929,11 +942,11 @@ END_OF_LUA
         my ( $resolution_name, $closure ) =
           @{ $rule_resolutions->[$resolution_rule] };
         if ($trace_actions) {
-            my $lhs_name = $slg->g1_symbol_name($lhs_id);
+            my $lhs_name = $slg->g1_symbol_display($lhs_id);
             say {$trace_file_handle}
               qq{Nulled symbol "$lhs_name" },
               qq{ resolved to "$resolution_name" from rule },
-              $slg->brief_rule($resolution_rule)
+              $slg->g1_rule_show($resolution_rule)
               or Marpa::R3::exception('print to trace handle failed');
         } ## end if ($trace_actions)
         $null_symbol_closures[$lhs_id] = $resolution_rule;
@@ -978,7 +991,7 @@ END_OF_LUA
                 if ( not $allowed_semantics->{$semantics} ) {
                     Marpa::R3::exception(
                         q{Unknown semantics for lexeme },
-                        $slg->g1_symbol_name($lexeme_id),
+                        $slg->g1_symbol_display($lexeme_id),
                         "\n",
                         qq{    Semantics were specified as "$semantics"\n}
                     );
@@ -995,7 +1008,7 @@ END_OF_LUA
                   if $blessing =~ /\A [[:alpha:]] [:\w]* \z /xms;
                 Marpa::R3::exception(
                     q{Unknown blessing for lexeme },
-                    $slg->g1_symbol_name($lexeme_id),
+                    $slg->g1_symbol_display($lexeme_id),
                     "\n",
                     qq{    Blessing as specified as "$blessing"\n}
                 );
@@ -1045,7 +1058,7 @@ END_OF_LUA
     } ## end NULLING_SYMBOL: for my $nulling_symbol ( 0 .. $#{$null_values} )
 
     my @work_list = ();
-  RULE: for my $irlid ( $slg->rule_ids() ) {
+    RULE: for (my $iter = $slg->g1_rule_ids_gen(); defined ( my $irlid = $iter->());) {
 
         my $semantics = $semantics_by_irlid[$irlid];
         my $blessing  = $blessing_by_irlid[$irlid];
@@ -1102,9 +1115,9 @@ END_OF_LUA
         local g1g = slg.g1
         local is_sequence_rule = g1g:sequence_min(irlid) and 1 or 0
         local irl = slg.g1.irls[irlid]
-        local xbnf = irl.xbnf
+        local xpr = irl.xpr
         local is_discard_sequence = false
-        if xbnf and xbnf.discard_separation and is_sequence_rule then
+        if xpr and xpr.discard_separation and is_sequence_rule then
             is_discard_sequence = true
         end
         return g1g:rule_length(irlid), is_sequence_rule, is_discard_sequence
@@ -1142,7 +1155,7 @@ END_OF_LUA
                 last CHECK_TYPE if not defined $thingy_ref;
                 my $ref_type = Scalar::Util::reftype $thingy_ref;
                 if ( $ref_type eq q{} ) {
-                    my $rule_desc = $slg->rule_show($irlid);
+                    my $rule_desc = $slg->g1_rule_show($irlid);
                     Marpa::R3::exception(
                         qq{An action resolved to a scalar.\n},
                         qq{  This is not allowed.\n},
@@ -1160,7 +1173,7 @@ END_OF_LUA
                     last CHECK_TYPE;
                 } ## end if ( $ref_type eq 'CODE' )
 
-                my $rule_desc = $slg->rule_show($irlid);
+                my $rule_desc = $slg->g1_rule_show($irlid);
                 Marpa::R3::exception(
                     qq{Constant action is not of an allowed type.\n},
                     qq{  It was of type reference to $ref_type.\n},
@@ -1212,7 +1225,7 @@ END_OF_LUA
                     my $original_semantics = $semantics_by_irlid[$irlid];
                     Marpa::R3::exception(
                         q{Impossible semantics for empty rule: },
-                        $slg->brief_rule($irlid),
+                        $slg->g1_rule_show($irlid),
                         "\n",
 qq{    Semantics were specified as "$original_semantics"\n}
                     );
@@ -1223,7 +1236,7 @@ qq{    Semantics were specified as "$original_semantics"\n}
                     my $original_semantics = $semantics_by_irlid[$irlid];
                     Marpa::R3::exception(
                         q{Impossible semantics for rule: },
-                        $slg->brief_rule($irlid),
+                        $slg->g1_rule_show($irlid),
                         "\n",
 qq{    Semantics were specified as "$original_semantics"\n}
                     );
@@ -1297,7 +1310,9 @@ END_OF_LUA
 
                 if ( $result_descriptor eq 'name' ) {
                     if ( defined $irlid ) {
-                        my $name = $slg->rule_name($irlid);
+                        my $production_id =
+                          $slg->g1_rule_to_production_id($irlid);
+                        my $name = $slg->production_name($production_id);
                         push @push_ops, $op_lua, $op_push_constant_key, \$name;
                         next RESULT_DESCRIPTOR;
                     }
@@ -1411,7 +1426,7 @@ END_OF_LUA
         # RHS of the start rule --
         # so copy them.
 
-        my $start_symbol_id = $slg->symbol_by_name('[:start]');
+        my $start_symbol_id = $slg->g1_symbol_by_name('[:start:]');
 
         my ($symbol_is_nullable) =
           $slg->call_by_tag( ( '@' . __FILE__ . ':' . __LINE__ ),
@@ -1424,8 +1439,8 @@ END_OF_LUA
         last SLR_NULLING_GRAMMAR_HACK if not $symbol_is_nullable;
 
         my $start_rhs_symbol_id;
-      RULE: for my $irlid ( $slg->rule_ids() ) {
-            my ( $lhs, $rhs0 ) = $slg->irl_isyids($irlid);
+      RULE: for (my $iter = $slg->g1_rule_ids_gen(); defined ( my $irlid = $iter->());) {
+            my ( $lhs, $rhs0 ) = $slg->g1_rule_expand($irlid);
             if ( $start_symbol_id == $lhs ) {
                 $start_rhs_symbol_id = $rhs0;
                 last RULE;
@@ -1522,6 +1537,7 @@ END_OF_LUA
     return if not defined $result;
 
     local $Marpa::R3::Context::rule = undef;
+    local $Marpa::R3::Context::irlid = undef;
     local $Marpa::R3::Context::slr  = $slr;
     local $Marpa::R3::Context::slg =
       $slr->[Marpa::R3::Internal::Scanless::R::SLG];
@@ -1634,7 +1650,10 @@ END_OF_LUA
                 };
 
                 $eval_ok = eval {
-                    local $Marpa::R3::Context::rule = $null_values->[$token_id];
+                    my $irlid = $null_values->[$token_id];
+                    local $Marpa::R3::Context::irlid = $irlid;
+                    local $Marpa::R3::Context::production_id =
+                      $slg->g1_rule_to_production_id($irlid);
                     $result = $value_ref->( $semantics_arg0, [] );
                     1;
                 };
@@ -1650,7 +1669,7 @@ END_OF_LUA
                         warnings    => \@warnings,
                         where       => 'computing value',
                         long_where  => 'Computing value for null symbol: '
-                          . $slg->g1_symbol_name($token_id),
+                          . $slg->g1_symbol_display($token_id),
                     }
                 );
             } ## end if ( not $eval_ok or @warnings )
@@ -1673,8 +1692,8 @@ END_OF_LUA
         } ## end if ( $value_type eq 'MARPA_STEP_NULLING_SYMBOL' )
 
         if ( $value_type eq 'MARPA_STEP_RULE' ) {
-            my ( $rule_id, $values ) = @value_data;
-            my $closure = $rule_closures->[$rule_id];
+            my ( $irlid, $values ) = @value_data;
+            my $closure = $rule_closures->[$irlid];
 
             next STEP if not defined $closure;
             my $result;
@@ -1685,7 +1704,8 @@ END_OF_LUA
                 local $SIG{__WARN__} = sub {
                     push @warnings, [ $_[0], ( caller 0 ) ];
                 };
-                local $Marpa::R3::Context::rule = $rule_id;
+                local $Marpa::R3::Context::irlid = $irlid;
+                local $Marpa::R3::Context::production_id = $slg->g1_rule_to_production_id($irlid);
 
                 # say STDERR "Before tree ops: ", Data::Dumper::Dumper($values);
                 $values = do_tree_ops( $slr, $values );
@@ -1706,7 +1726,7 @@ END_OF_LUA
                             warnings    => \@warnings,
                             where       => 'computing value',
                             long_where  => 'Computing value for rule: '
-                              . $slg->brief_rule($rule_id),
+                              . $slg->g1_rule_show($irlid),
                         }
                     );
                 } ## end if ( not $eval_ok or @warnings )
@@ -1727,7 +1747,7 @@ END_OF_LUA
 
             if ($trace_values) {
                 say {$trace_file_handle}
-                  trace_stack_1( $slr, $values, $rule_id )
+                  trace_stack_1( $slr, $values, $irlid )
                   or Marpa::R3::exception('Could not print to trace file');
                 print {$trace_file_handle}
                   'Calculated and pushed value: ',
@@ -1805,7 +1825,7 @@ END_OF_LUA
     }
     my $token_name;
     if ( defined $token_id ) {
-        $token_name = $slg->g1_symbol_name($token_id);
+        $token_name = $slg->g1_symbol_display($token_id);
     }
 
     print {$trace_file_handle}
@@ -1840,7 +1860,7 @@ END_OF_LUA
     return 'Popping ', $argc,
       ' values to evaluate ',
       $slr->and_node_tag($and_node_id),
-      ', rule: ', $slg->brief_rule($rule_id);
+      ', rule: ', $slg->g1_rule_show($rule_id);
 
 } ## end sub trace_stack_1
 
@@ -1869,9 +1889,9 @@ sub trace_op {
     local g1g = slr.slg.g1
     local or_node_id = t:_nook_or_node(nook_ix)
     local choice = t:_nook_choice(nook_ix)
-    local trace_irl_id = b:_or_node_irl(or_node_id)
+    local trace_irl_id = b:_or_node_nrl(or_node_id)
     local or_node_position = b:_or_node_position(or_node_id)
-    local irl_length = g1g:_irl_length(trace_irl_id)
+    local irl_length = g1g:_nrl_length(trace_irl_id)
     if irl_length ~= or_node_position then
         return 'return'
     end
@@ -1879,8 +1899,8 @@ sub trace_op {
         '', nook_ix, or_node_id, choice,
             o:_and_order_get(or_node_id, choice), 
             trace_irl_id,
-            g1g:_irl_is_virtual_rhs(trace_irl_id),
-            g1g:_irl_is_virtual_lhs(trace_irl_id),
+            g1g:_nrl_is_virtual_rhs(trace_irl_id),
+            g1g:_nrl_is_virtual_lhs(trace_irl_id),
             g1g:_real_symbol_count(trace_irl_id)
 END_OF_LUA
 
